@@ -36,6 +36,9 @@
 #include <glib/gi18n.h>
 #include <gdk/gdk.h>
 #include <gst/tag/tag.h>
+#if GST_CHECK_VERSION(0,10,25)
+#include <gst/interfaces/streamvolume.h>
+#endif
 #include <gst/pbutils/pbutils.h>
 
 #include "rb-debug.h"
@@ -88,6 +91,8 @@ struct _RBPlayerGstPrivate
 	char *uri;
 	gpointer stream_data;
 	GDestroyNotify stream_data_destroy;
+	gpointer next_stream_data;
+	GDestroyNotify next_stream_data_destroy;
 
 	GstElement *playbin;
 	GstElement *audio_sink;
@@ -98,14 +103,15 @@ struct _RBPlayerGstPrivate
 
 	gboolean stream_change_pending;
 	gboolean current_track_finishing;
-	gboolean fake_playing_position;
+	gboolean playbin_stream_changing;
 
 	gboolean emitted_error;
 
+	GList *stream_tags;
+
 	gint volume_changed;
 	gint volume_applied;
 	float cur_volume;
-	float replaygain_scale;
 
 	guint tick_timeout_id;
 
@@ -117,19 +123,25 @@ struct _RBPlayerGstPrivate
 	GstElement *filterbin;
 };
 
-static gboolean
-tick_timeout (RBPlayerGst *mp)
+static void
+_destroy_stream_data (RBPlayerGst *player)
 {
-	if (mp->priv->playing) {
-		_rb_player_emit_tick (RB_PLAYER (mp),
-				      mp->priv->stream_data,
-				      rb_player_get_time (RB_PLAYER (mp)),
-				      -1);
-		mp->priv->fake_playing_position = FALSE;
+	if (player->priv->stream_data && player->priv->stream_data_destroy) {
+		player->priv->stream_data_destroy (player->priv->stream_data);
 	}
-	return TRUE;
+	player->priv->stream_data = NULL;
+	player->priv->stream_data_destroy = NULL;
 }
 
+static void
+_destroy_next_stream_data (RBPlayerGst *player)
+{
+	if (player->priv->next_stream_data && player->priv->next_stream_data_destroy) {
+		player->priv->next_stream_data_destroy (player->priv->next_stream_data);
+	}
+	player->priv->next_stream_data = NULL;
+	player->priv->next_stream_data_destroy = NULL;
+}
 
 static void
 about_to_finish_cb (GstElement *playbin, RBPlayerGst *player)
@@ -149,13 +161,26 @@ about_to_finish_cb (GstElement *playbin, RBPlayerGst *player)
 	/* emit EOS now and hope we get something to play */
 	player->priv->current_track_finishing = TRUE;
 
-	_rb_player_emit_eos (RB_PLAYER (player), player->priv->stream_data);
+	_rb_player_emit_eos (RB_PLAYER (player), player->priv->stream_data, TRUE);
 }
 
 static gboolean
 emit_volume_changed_idle (RBPlayerGst *player)
 {
-	_rb_player_emit_volume_changed (RB_PLAYER (player), player->priv->cur_volume);
+	double vol;
+
+#if GST_CHECK_VERSION(0,10,25)
+	if (gst_element_implements_interface (player->priv->playbin, GST_TYPE_STREAM_VOLUME)) {
+		vol = gst_stream_volume_get_volume (GST_STREAM_VOLUME (player->priv->playbin),
+						    GST_STREAM_VOLUME_FORMAT_CUBIC);
+	} else {
+		vol = player->priv->cur_volume;
+	}
+#else
+	vol = player->priv->cur_volume;
+#endif
+
+	_rb_player_emit_volume_changed (RB_PLAYER (player), vol);
 	return FALSE;
 }
 
@@ -195,6 +220,35 @@ process_tag (const GstTagList *list, const gchar *tag, RBPlayerGst *player)
 	}
 }
 
+static void
+emit_playing_stream_and_tags (RBPlayerGst *player, gboolean track_change)
+{
+	GList *t;
+
+	if (track_change) {
+		/* swap stream data */
+		_destroy_stream_data (player);
+		player->priv->stream_data = player->priv->next_stream_data;
+		player->priv->stream_data_destroy = player->priv->next_stream_data_destroy;
+		player->priv->next_stream_data = NULL;
+		player->priv->next_stream_data_destroy = NULL;
+	}
+
+	_rb_player_emit_playing_stream (RB_PLAYER (player), player->priv->stream_data);
+
+	/* process any tag lists we received while starting the stream */
+	for (t = player->priv->stream_tags; t != NULL; t = t->next) {
+		GstTagList *tags;
+
+		tags = (GstTagList *)t->data;
+		rb_debug ("processing buffered taglist");
+		gst_tag_list_foreach (tags, (GstTagForeachFunc) process_tag, player);
+		gst_tag_list_free (tags);
+	}
+	g_list_free (player->priv->stream_tags);
+	player->priv->stream_tags = NULL;
+}
+
 static void
 handle_missing_plugin_message (RBPlayerGst *player, GstMessage *message)
 {
@@ -278,16 +332,19 @@ bus_cb (GstBus *bus, GstMessage *message, RBPlayerGst *mp)
 	}
 
 	case GST_MESSAGE_EOS:
-		rb_debug ("got EOS.. why haven't you told me what to do yet?");
-		_rb_player_emit_eos (RB_PLAYER (mp), mp->priv->stream_data);
+		_rb_player_emit_eos (RB_PLAYER (mp), mp->priv->stream_data, FALSE);
 		break;
 
 	case GST_MESSAGE_TAG: {
 		GstTagList *tags;
 		gst_message_parse_tag (message, &tags);
 
-		gst_tag_list_foreach (tags, (GstTagForeachFunc) process_tag, mp);
-		gst_tag_list_free (tags);
+		if (mp->priv->stream_change_pending) {
+			mp->priv->stream_tags = g_list_append (mp->priv->stream_tags, tags);
+		} else {
+			gst_tag_list_foreach (tags, (GstTagForeachFunc) process_tag, mp);
+			gst_tag_list_free (tags);
+		}
 		break;
 	}
 
@@ -330,8 +387,14 @@ bus_cb (GstBus *bus, GstMessage *message, RBPlayerGst *mp)
 		break;
 
 	case GST_MESSAGE_ELEMENT:
+		structure = gst_message_get_structure (message);
 		if (gst_is_missing_plugin_message (message)) {
 			handle_missing_plugin_message (mp, message);
+		} else if (mp->priv->playbin_stream_changing &&
+			   gst_structure_has_name (structure, "playbin2-stream-changed")) {
+			rb_debug ("got playbin2-stream-changed message");
+			mp->priv->playbin_stream_changing = FALSE;
+			emit_playing_stream_and_tags (mp, TRUE);
 		}
 		break;
 
@@ -423,24 +486,10 @@ construct_pipeline (RBPlayerGst *mp, GError **error)
 		GstPad *pad;
 		GList *l;
 		GstElement *queue;
-		GstElement *audioconvert;
 		GstPad *ghostpad;
 
-		/* setup filterbin,and insert the leading audioconvert */
-		mp->priv->filterbin = gst_bin_new (NULL);
-		audioconvert = gst_element_factory_make ("audioconvert", NULL);
-		gst_bin_add (GST_BIN (mp->priv->filterbin), audioconvert);
-
-		/* ghost it to the bin */
-		pad = gst_element_get_pad (audioconvert, "sink");
-		ghostpad = gst_ghost_pad_new ("sink", pad);
-		gst_element_add_pad (mp->priv->filterbin, ghostpad);
-		gst_object_unref (pad);
-
-		pad = gst_element_get_pad (audioconvert, "src");
-		ghostpad = gst_ghost_pad_new ("src", pad);
-		gst_element_add_pad (mp->priv->filterbin, ghostpad);
-		gst_object_unref (pad);
+		/* setup filterbin */
+		mp->priv->filterbin = rb_gst_create_filter_bin ();
 
 		/* set up the sinkbin with its tee element */
 		mp->priv->sinkbin = gst_bin_new (NULL);
@@ -458,7 +507,6 @@ construct_pipeline (RBPlayerGst *mp, GError **error)
 
 		g_object_set (G_OBJECT (mp->priv->playbin), "audio-sink", mp->priv->sinkbin, NULL);
 
-
 		/* add any tees and filters that were waiting for us */
 		for (l = mp->priv->waiting_tees; l != NULL; l = g_list_next (l)) {
 			rb_player_gst_tee_add_tee (RB_PLAYER_GST_TEE (mp), GST_ELEMENT (l->data));
@@ -486,7 +534,6 @@ construct_pipeline (RBPlayerGst *mp, GError **error)
 		mp->priv->cur_volume = 1.0;
 	if (mp->priv->cur_volume < 0.0)
 		mp->priv->cur_volume = 0;
-	mp->priv->replaygain_scale = 1.0f;
 
 	rb_debug ("pipeline construction complete");
 	return TRUE;
@@ -619,6 +666,8 @@ set_state_and_wait (RBPlayerGst *player, GstState target, GError **error)
 			bus_cb (bus, message, player);
 			break;
 		}
+
+		gst_message_unref (message);
 	}
 
 	if (result == FALSE && error != NULL && *error == NULL) {
@@ -631,16 +680,32 @@ set_state_and_wait (RBPlayerGst *player, GstState target, GError **error)
 	return result;
 }
 
-static void
-_destroy_stream_data (RBPlayerGst *player)
+static gboolean
+tick_timeout (RBPlayerGst *mp)
 {
-	if (player->priv->stream_data && player->priv->stream_data_destroy) {
-		player->priv->stream_data_destroy (player->priv->stream_data);
+	if (mp->priv->playing) {
+		gint64 position;
+
+		position = rb_player_get_time (RB_PLAYER (mp));
+
+		/* if we don't have stream-changed messages, do the track change when
+		 * the playback position is less than one second into the current track,
+		 * which pretty much has to be the new one.
+		 */
+		if (mp->priv->playbin_stream_changing && (position < GST_SECOND)) {
+			emit_playing_stream_and_tags (mp, TRUE);
+			mp->priv->playbin_stream_changing = FALSE;
+		}
+
+		_rb_player_emit_tick (RB_PLAYER (mp),
+				      mp->priv->stream_data,
+				      position,
+				      -1);
 	}
-	player->priv->stream_data = NULL;
-	player->priv->stream_data_destroy = NULL;
+	return TRUE;
 }
 
+
 static gboolean
 impl_close (RBPlayer *player, const char *uri, GError **error)
 {
@@ -651,26 +716,19 @@ impl_close (RBPlayer *player, const char *uri, GError **error)
 		return TRUE;
 	}
 
-	if (mp->priv->current_track_finishing) {
-		/* this is a bit of a hack.
-		 * about-to-finish should result in either a track change or nothing.
-		 */
-		rb_debug ("not closing in about-to-finish callback");
-		mp->priv->current_track_finishing = FALSE;
-		return TRUE;
-	}
-
 	mp->priv->playing = FALSE;
 	mp->priv->buffering = FALSE;
+	mp->priv->current_track_finishing = FALSE;
 
 	_destroy_stream_data (mp);
+	if (uri == NULL) {
+		_destroy_next_stream_data (mp);
+	}
 	g_free (mp->priv->uri);
 	g_free (mp->priv->prev_uri);
 	mp->priv->uri = NULL;
 	mp->priv->prev_uri = NULL;
 
-	mp->priv->replaygain_scale = 1.0f;
-
 	if (mp->priv->tick_timeout_id != 0) {
 		g_source_remove (mp->priv->tick_timeout_id);
 		mp->priv->tick_timeout_id = 0;
@@ -703,12 +761,12 @@ impl_open (RBPlayer *player,
 	}
 
 	rb_debug ("setting new uri to %s", uri);
-	_destroy_stream_data (mp);
+	_destroy_next_stream_data (mp);
 	g_free (mp->priv->prev_uri);
 	mp->priv->prev_uri = mp->priv->uri;
 	mp->priv->uri = g_strdup (uri);
-	mp->priv->stream_data = stream_data;
-	mp->priv->stream_data_destroy = stream_data_destroy;
+	mp->priv->next_stream_data = stream_data;
+	mp->priv->next_stream_data_destroy = stream_data_destroy;
 	mp->priv->emitted_error = FALSE;
 	mp->priv->stream_change_pending = TRUE;
 
@@ -730,7 +788,15 @@ set_playbin_volume (RBPlayerGst *player, float volume)
 	 * we still get another one anyway.
 	 */
 	g_signal_handlers_block_by_func (player->priv->playbin, volume_notify_cb, player);
+#if GST_CHECK_VERSION(0,10,25)
+	if (gst_element_implements_interface (player->priv->playbin, GST_TYPE_STREAM_VOLUME))
+		gst_stream_volume_set_volume (GST_STREAM_VOLUME (player->priv->playbin),
+					      GST_STREAM_VOLUME_FORMAT_CUBIC, volume);
+	else
+		g_object_set (player->priv->playbin, "volume", volume, NULL);
+#else
 	g_object_set (player->priv->playbin, "volume", volume, NULL);
+#endif
 	g_signal_handlers_unblock_by_func (player->priv->playbin, volume_notify_cb, player);
 }
 
@@ -739,23 +805,21 @@ impl_play (RBPlayer *player, RBPlayerPlayType play_type, gint64 crossfade, GErro
 {
 	RBPlayerGst *mp = RB_PLAYER_GST (player);
 	gboolean result;
+	gboolean track_change = TRUE;
 
 	g_return_val_if_fail (mp->priv->playbin != NULL, FALSE);
 
 	if (mp->priv->stream_change_pending == FALSE) {
 		rb_debug ("no stream change pending, just restarting playback");
 		result = set_state_and_wait (mp, GST_STATE_PLAYING, error);
+		track_change = FALSE;
 
 	} else if (mp->priv->current_track_finishing) {
 		rb_debug ("current track finishing -> just setting URI on playbin");
 		g_object_set (mp->priv->playbin, "uri", mp->priv->uri, NULL);
 		result = TRUE;
 
-		/* since we don't know exactly when the change occurs, pretend the playing position
-		 * is 0 until the next tick.  otherwise we sometimes get the playing position from the
-		 * previous track.
-		 */
-		mp->priv->fake_playing_position = TRUE;
+		mp->priv->playbin_stream_changing = TRUE;
 
 	} else {
 		gboolean reused = FALSE;
@@ -795,7 +859,9 @@ impl_play (RBPlayer *player, RBPlayerPlayType play_type, gint64 crossfade, GErro
 		mp->priv->buffering = FALSE;
 		mp->priv->playing = TRUE;
 
-		_rb_player_emit_playing_stream (RB_PLAYER (mp), mp->priv->stream_data);
+		if (mp->priv->playbin_stream_changing == FALSE) {
+			emit_playing_stream_and_tags (mp, track_change);
+		}
 
 		if (mp->priv->tick_timeout_id == 0) {
 			mp->priv->tick_timeout_id =
@@ -814,13 +880,12 @@ impl_play (RBPlayer *player, RBPlayerPlayType play_type, gint64 crossfade, GErro
 			e = rb_player_gst_find_element_with_property (mp->priv->audio_sink, "volume");
 			if (e != NULL) {
 				mp->priv->volume_applied = 1;
+				gst_object_unref (e);
 			}
-			gst_object_unref (e);
 
 			if (mp->priv->volume_applied < mp->priv->volume_changed) {
-				float volume = mp->priv->cur_volume * mp->priv->replaygain_scale;
-				rb_debug ("applying initial volume: %f", volume);
-				set_playbin_volume (mp, volume);
+				rb_debug ("applying initial volume: %f", mp->priv->cur_volume);
+				set_playbin_volume (mp, mp->priv->cur_volume);
 			}
 
 			mp->priv->volume_applied = mp->priv->volume_changed;
@@ -862,50 +927,6 @@ impl_playing (RBPlayer *player)
 	return mp->priv->playing;
 }
 
-static void
-impl_set_replaygain (RBPlayer *player,
-		     const char *uri,
-		     double track_gain,
-		     double track_peak,
-		     double album_gain,
-		     double album_peak)
-{
-	RBPlayerGst *mp = RB_PLAYER_GST (player);
-	double scale;
-	double gain = 0;
-	double peak = 0;
-
-	if (album_gain != 0)
-		gain = album_gain;
-	else
-		gain = track_gain;
-
-	if (gain == 0)
-		return;
-
-	scale = pow (10., gain / 20);
-
-	/* anti clip */
-	if (album_peak != 0)
-		peak = album_peak;
-	else
-		peak = track_peak;
-
-	if (peak != 0 && (scale * peak) > 1)
-		scale = 1.0 / peak;
-
-	/* For security */
-	if (scale > 15)
-		scale = 15;
-
-	rb_debug ("Scale : %f New volume : %f", scale, mp->priv->cur_volume * scale);
-	mp->priv->replaygain_scale = scale;
-
-	if (mp->priv->playbin != NULL) {
-		g_object_set (mp->priv->playbin, "volume", mp->priv->cur_volume * scale, NULL);
-	}
-}
-
 static void
 impl_set_volume (RBPlayer *player,
 		 float volume)
@@ -915,7 +936,7 @@ impl_set_volume (RBPlayer *player,
 
 	mp->priv->volume_changed++;
 	if (mp->priv->volume_applied > 0) {
-		set_playbin_volume (mp, volume * mp->priv->replaygain_scale);
+		set_playbin_volume (mp, volume);
 		mp->priv->volume_applied = mp->priv->volume_changed;
 	} else {
 		/* volume will be applied in the first call to impl_play */
@@ -991,10 +1012,7 @@ impl_get_time (RBPlayer *player)
 {
 	RBPlayerGst *mp = RB_PLAYER_GST (player);
 
-	if (mp->priv->fake_playing_position) {
-		/* track transition occurring, so pretend we're at the start of the new one */
-		return 0;
-	} else if (mp->priv->playbin != NULL) {
+	if (mp->priv->playbin != NULL) {
 		gint64 position = -1;
 		GstFormat fmt = GST_FORMAT_TIME;
 
@@ -1005,61 +1023,29 @@ impl_get_time (RBPlayer *player)
 	}
 }
 
+static gboolean
+need_pad_blocking (RBPlayerGst *mp)
+{
+	return (mp->priv->playing || (mp->priv->uri != NULL));
+}
+
 static gboolean
 impl_add_tee (RBPlayerGstTee *player, GstElement *element)
 {
-	RBPlayerGst *mp;
-	GstElement *queue, *audioconvert, *bin;
-	GstPad *pad, *ghostpad;
-
-	mp = RB_PLAYER_GST (player);
+	RBPlayerGst *mp = RB_PLAYER_GST (player);
 
 	if (mp->priv->tee == NULL) {
 		mp->priv->waiting_tees = g_list_prepend (mp->priv->waiting_tees, element);
 		return TRUE;
 	}
 
-	if (mp->priv->playing) {
-		GError *error = NULL;
-		if (set_state_and_wait (mp, GST_STATE_PAUSED, &error) == FALSE) {
-			g_warning ("Failed to pause pipeline before tee insertion: %s", error->message);
-			g_error_free (error);
-			return FALSE;
-		}
-	}
-
-	bin = gst_bin_new (NULL);
-	queue = gst_element_factory_make ("queue", NULL);
-	audioconvert = gst_element_factory_make ("audioconvert", NULL);
-
-	/* set up the element's containing bin */
-	gst_bin_add_many (GST_BIN (bin), queue, audioconvert, element, NULL);
-	gst_bin_add (GST_BIN (mp->priv->sinkbin), bin);
-	gst_element_link_many (queue, audioconvert, element, NULL);
-
-	/* link it to the tee */
-	pad = gst_element_get_pad (queue, "sink");
-	ghostpad = gst_ghost_pad_new ("sink", pad);
-	gst_element_add_pad (bin, ghostpad);
-	gst_object_unref (pad);
-
-	gst_element_link (mp->priv->tee, bin);
-
-	if (mp->priv->playing)
-		gst_element_set_state (mp->priv->playbin, GST_STATE_PLAYING);
-
-	_rb_player_gst_tee_emit_tee_inserted (player, element);
-
-	return TRUE;
+	return rb_gst_add_tee (RB_PLAYER (player), mp->priv->tee, element, need_pad_blocking (mp));
 }
 
 static gboolean
 impl_remove_tee (RBPlayerGstTee *player, GstElement *element)
 {
-	RBPlayerGst *mp;
-	GstElement *bin;
-
-	mp = RB_PLAYER_GST (player);
+	RBPlayerGst *mp = RB_PLAYER_GST (player);
 
 	if (mp->priv->tee == NULL) {
 		gst_object_sink (element);
@@ -1067,137 +1053,25 @@ impl_remove_tee (RBPlayerGstTee *player, GstElement *element)
 		return TRUE;
 	}
 
-	_rb_player_gst_tee_emit_tee_pre_remove (player, element);
-
-	if (mp->priv->playing) {
-		GError *error = NULL;
-		if (set_state_and_wait (mp, GST_STATE_PAUSED, &error) == FALSE) {
-			g_warning ("Failed to pause pipeline before tee removal: %s", error->message);
-			g_error_free (error);
-			return FALSE;
-		}
-	}
-
-	/* get the containing bin and unlink it */
-	bin = GST_ELEMENT (gst_element_get_parent (element));
-
-	gst_element_set_state (bin, GST_STATE_NULL);
-
-	gst_bin_remove (GST_BIN (mp->priv->sinkbin), bin);
-	gst_object_unref (bin);
-
-	if (mp->priv->playing)
-		gst_element_set_state (mp->priv->playbin, GST_STATE_PLAYING);
-
-	return TRUE;
+	return rb_gst_remove_tee (RB_PLAYER (mp), mp->priv->tee, element, need_pad_blocking (mp));
 }
 
 static gboolean
 impl_add_filter (RBPlayerGstFilter *player, GstElement *element)
 {
-	RBPlayerGst *mp;
-	GstElement *audioconvert, *bin;
-	GstPad *ghostpad, *realpad;
-	GstPad *binsinkpad, *binsrcpad;
-	gpointer element_sink_pad;
-	GstIterator *sink_pads;
-	gboolean sink_pad_found, stop_scan;
-	GstPadLinkReturn link;
-
-	mp = RB_PLAYER_GST (player);
+	RBPlayerGst *mp = RB_PLAYER_GST (player);
 
 	if (mp->priv->filterbin == NULL) {
 		mp->priv->waiting_filters = g_list_prepend (mp->priv->waiting_filters, element);
 		return TRUE;
 	}
-
-	if (mp->priv->playing) {
-		GError *error = NULL;
-		if (set_state_and_wait (mp, GST_STATE_PAUSED, &error) == FALSE) {
-			g_warning ("Failed to pause pipeline before filter insertion: %s", error->message);
-			g_error_free (error);
-			return FALSE;
-		}
-	}
-
-	bin = gst_bin_new (NULL);
-	audioconvert = gst_element_factory_make ("audioconvert", NULL);
-
-	/* set up the element's containing bin */
-	rb_debug ("adding element %p and audioconvert to bin", element);
-	gst_bin_add_many (GST_BIN (bin), element, audioconvert, NULL);
-	gst_element_link_many (element, audioconvert, NULL);
-
-	/* ghost to the bin */
-	/* retrieve the first unliked source pad */
-	sink_pad_found = FALSE;
-	stop_scan = FALSE;
-	sink_pads = gst_element_iterate_sink_pads (element);
-	while (!sink_pad_found && !stop_scan) {
-		gpointer *esp_pointer = &element_sink_pad; /* stop type-punning warnings */
-		switch (gst_iterator_next (sink_pads, esp_pointer)) {
-			case GST_ITERATOR_OK:
-				sink_pad_found = !gst_pad_is_linked (GST_PAD(element_sink_pad));
-				break;
-			case GST_ITERATOR_RESYNC:
-				gst_iterator_resync (sink_pads);
-				break;
-			case GST_ITERATOR_ERROR:
-			case GST_ITERATOR_DONE:
-				stop_scan = TRUE;
-				break;
-		}
-	}
-	gst_iterator_free (sink_pads);
-
-	if (!sink_pad_found) {
-		g_warning ("Could not find a free sink pad on filter");
-		return FALSE;
-	}
-
-	binsinkpad = gst_ghost_pad_new ("sink", GST_PAD (element_sink_pad));
-	gst_element_add_pad (bin, binsinkpad);
-
-	realpad = gst_element_get_pad (audioconvert, "src");
-	binsrcpad = gst_ghost_pad_new ("src", realpad);
-	gst_element_add_pad (bin, binsrcpad);
-	gst_object_unref (realpad);
-
-	/* replace the filter chain ghost with the new bin */
-	gst_bin_add (GST_BIN (mp->priv->filterbin), bin);
-
-	ghostpad = gst_element_get_pad (mp->priv->filterbin, "src");
-	realpad = gst_ghost_pad_get_target (GST_GHOST_PAD (ghostpad));
-	gst_ghost_pad_set_target (GST_GHOST_PAD (ghostpad), binsrcpad);
-	gst_object_unref (ghostpad);
-
-	link = gst_pad_link (realpad, binsinkpad);
-	gst_object_unref (realpad);
-	if (link != GST_PAD_LINK_OK) {
-		g_warning ("could not link new filter into pipeline");
-		return FALSE;
-	}
-
-	if (mp->priv->playing)
-		gst_element_set_state (mp->priv->playbin, GST_STATE_PLAYING);
-
-	_rb_player_gst_filter_emit_filter_inserted (player, element);
-
-	return TRUE;
+	return rb_gst_add_filter (RB_PLAYER (mp), mp->priv->filterbin, element, need_pad_blocking (mp));
 }
 
 static gboolean
 impl_remove_filter (RBPlayerGstFilter *player, GstElement *element)
 {
-	RBPlayerGst *mp;
-	GstPad *mypad;
-	GstPad *prevpad, *nextpad;
-	GstPad *ghostpad;
-	GstPad *targetpad;
-	GstElement *bin;
-	gboolean result = TRUE;
-
-	mp = RB_PLAYER_GST (player);
+	RBPlayerGst *mp = RB_PLAYER_GST (player);
 
 	if (mp->priv->filterbin == NULL) {
 		gst_object_sink (element);
@@ -1205,58 +1079,7 @@ impl_remove_filter (RBPlayerGstFilter *player, GstElement *element)
 		return TRUE;
 	}
 
-	_rb_player_gst_filter_emit_filter_pre_remove (player, element);
-
-	if (mp->priv->playing) {
-		GError *error = NULL;
-		if (set_state_and_wait (mp, GST_STATE_PAUSED, &error) == FALSE) {
-			g_warning ("Failed to pause pipeline before filter removal: %s", error->message);
-			g_error_free (error);
-			return FALSE;
-		}
-	}
-
-	/* get the containing bin and unlink it */
-	bin = GST_ELEMENT (gst_element_get_parent (element));
-
-	gst_element_set_state (bin, GST_STATE_NULL);
-
-	mypad = gst_element_get_pad (bin, "sink");
-	prevpad = gst_pad_get_peer (mypad);
-	gst_pad_unlink (prevpad, mypad);
-	gst_object_unref (mypad);
-
-	ghostpad = gst_element_get_pad (bin, "src");
-	nextpad = gst_element_get_pad (mp->priv->filterbin, "src");
-
-	targetpad = gst_ghost_pad_get_target (GST_GHOST_PAD (nextpad));
-	if (targetpad == ghostpad) {
-		/* we are at the end of the filter chain, so redirect the ghostpad to the previous element */
-		gst_ghost_pad_set_target (GST_GHOST_PAD (nextpad), prevpad);
-	} else {
-		/* we are in the middle, so link the previous and next elements */
-		mypad = gst_element_get_pad (bin, "src");
-		gst_object_unref (nextpad);
-		nextpad = gst_pad_get_peer (mypad);
-		gst_pad_unlink (mypad, nextpad);
-		gst_object_unref (mypad);
-
-		if (gst_pad_link (prevpad, nextpad) != GST_PAD_LINK_OK)
-			result = FALSE;
-	}
-
-	gst_object_unref (nextpad);
-	gst_object_unref (prevpad);
-	gst_object_unref (ghostpad);
-	gst_object_unref (targetpad);
-
-	gst_bin_remove (GST_BIN (mp->priv->filterbin), bin);
-	gst_object_unref (bin);
-
-	if (mp->priv->playing)
-		gst_element_set_state (mp->priv->playbin, GST_STATE_PLAYING);
-
-	return result;
+	return rb_gst_remove_filter (RB_PLAYER (mp), mp->priv->filterbin, element, need_pad_blocking (mp));
 }
 
 static void
@@ -1386,7 +1209,6 @@ rb_player_init (RBPlayerIface *iface)
 	iface->playing = impl_playing;
 	iface->set_volume = impl_set_volume;
 	iface->get_volume = impl_get_volume;
-	iface->set_replaygain = impl_set_replaygain;
 	iface->seekable = impl_seekable;
 	iface->set_time = impl_set_time;
 	iface->get_time = impl_get_time;
