@@ -108,8 +108,10 @@ static void rb_mtp_plugin_device_added (LibHalContext *context, const char *udi)
 static void rb_mtp_plugin_device_removed (LibHalContext *context, const char *udi);
 static gboolean rb_mtp_plugin_setup_dbus_hal_connection (RBMtpPlugin *plugin);
 #endif
-static void rb_mtp_plugin_eject  (GtkAction *action, RBMtpPlugin *plugin);
-static void rb_mtp_plugin_rename (GtkAction *action, RBMtpPlugin *plugin);
+
+static void rb_mtp_plugin_eject  (GtkAction *action, RBSource *source);
+static void rb_mtp_plugin_rename (GtkAction *action, RBSource *source);
+static void rb_mtp_plugin_properties (GtkAction *action, RBSource *source);
 
 GType rb_mtp_src_get_type (void);
 GType rb_mtp_sink_get_type (void);
@@ -123,7 +125,10 @@ static GtkActionEntry rb_mtp_plugin_actions [] =
 	  G_CALLBACK (rb_mtp_plugin_eject) },
 	{ "MTPSourceRename", NULL, N_("_Rename"), NULL,
 	  N_("Rename MTP-device"),
-	  G_CALLBACK (rb_mtp_plugin_rename) }
+	  G_CALLBACK (rb_mtp_plugin_rename) },
+	{ "MTPSourceProperties", GTK_STOCK_PROPERTIES, N_("_Properties"), NULL,
+	  N_("Display device properties"),
+	  G_CALLBACK (rb_mtp_plugin_properties) }
 };
 
 static void
@@ -185,9 +190,10 @@ impl_activate (RBPlugin *bplugin, RBShell *shell)
 	plugin->action_group = gtk_action_group_new ("MTPActions");
 	gtk_action_group_set_translation_domain (plugin->action_group,
 						 GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (plugin->action_group,
-				      rb_mtp_plugin_actions, G_N_ELEMENTS (rb_mtp_plugin_actions),
-				      plugin);
+	_rb_action_group_add_source_actions (plugin->action_group,
+					     G_OBJECT (plugin->shell),
+					     rb_mtp_plugin_actions,
+					     G_N_ELEMENTS (rb_mtp_plugin_actions));
 	gtk_ui_manager_insert_action_group (uimanager, plugin->action_group, 0);
 	file = rb_plugin_find_file (bplugin, "mtp-ui.xml");
 	plugin->ui_merge_id = gtk_ui_manager_add_ui_from_file (uimanager, file, NULL);
@@ -282,51 +288,34 @@ impl_deactivate (RBPlugin *bplugin, RBShell *shell)
 }
 
 static void
-rb_mtp_plugin_eject (GtkAction *action, RBMtpPlugin *plugin)
+rb_mtp_plugin_eject (GtkAction *action, RBSource *source)
 {
-	RBSourceList *sourcelist = NULL;
-	RBSource *source = NULL;
-
-	g_object_get (G_OBJECT (plugin->shell),
-		      "selected-source", &source,
-		      NULL);
-	if ((source == NULL) || !RB_IS_MTP_SOURCE (source)) {
-		g_warning ("got MTPSourceEject action for non-mtp source");
-		if (source != NULL)
-			g_object_unref (source);
-		return;
-	}
-
-	g_object_get (plugin->shell, "sourcelist", &sourcelist, NULL);
-
+	g_return_if_fail (RB_IS_MTP_SOURCE (source));
 	rb_source_delete_thyself (source);
-
-	g_object_unref (sourcelist);
-	g_object_unref (source);
 }
 
 static void
-rb_mtp_plugin_rename (GtkAction *action, RBMtpPlugin *plugin)
+rb_mtp_plugin_rename (GtkAction *action, RBSource *source)
 {
-	RBSourceList *sourcelist = NULL;
-	RBSource *source = NULL;
+	RBShell *shell;
+	RBSourceList *sourcelist;
 
-	g_object_get (G_OBJECT (plugin->shell),
-		      "selected-source", &source,
-		      NULL);
-	if ((source == NULL) || !RB_IS_MTP_SOURCE (source)) {
-		g_warning ("got MTPSourceEject action for non-mtp source");
-		if (source != NULL)
-			g_object_unref (source);
-		return;
-	}
+	g_return_if_fail (RB_IS_MTP_SOURCE (source));
 
-	g_object_get (plugin->shell, "sourcelist", &sourcelist, NULL);
+	g_object_get (source, "shell", &shell, NULL);
+	g_object_get (shell, "sourcelist", &sourcelist, NULL);
 
 	rb_sourcelist_edit_source_name (sourcelist, source);
 
 	g_object_unref (sourcelist);
-	g_object_unref (source);
+	g_object_unref (shell);
+}
+
+static void
+rb_mtp_plugin_properties (GtkAction *action, RBSource *source)
+{
+	g_return_if_fail (RB_IS_MTP_SOURCE (source));
+	rb_media_player_source_show_properties (RB_MEDIA_PLAYER_SOURCE (source));
 }
 
 
@@ -338,9 +327,10 @@ source_deleted_cb (RBMtpSource *source, RBMtpPlugin *plugin)
 }
 
 static RBSource *
-create_source_device_cb (RBRemovableMediaManager *rmm, GObject *device, RBMtpPlugin *plugin)
+create_source_device_cb (RBRemovableMediaManager *rmm, GObject *device_obj, RBMtpPlugin *plugin)
 {
 	GUdevDeviceNumber device_number;
+	GUdevDevice *device = G_UDEV_DEVICE (device_obj);
 	int i;
 	int num_raw_devices;
 	const char *devnum_str;
@@ -348,28 +338,28 @@ create_source_device_cb (RBRemovableMediaManager *rmm, GObject *device, RBMtpPlu
 	LIBMTP_raw_device_t *raw_devices;
 
 	/* check subsystem == usb? */
-	if (g_strcmp0 (g_udev_device_get_subsystem (G_UDEV_DEVICE (device)), "usb") != 0) {
-		rb_debug ("this is not a USB device");
+	if (g_strcmp0 (g_udev_device_get_subsystem (device), "usb") != 0) {
+		rb_debug ("device %s is not a USB device", g_udev_device_get_name (device));
 		return NULL;
 	}
 
-	device_number = g_udev_device_get_device_number (G_UDEV_DEVICE (device));
+	device_number = g_udev_device_get_device_number (device);
 	if (device_number == 0) {
-		rb_debug ("can't get udev device number for this device");
+		rb_debug ("can't get udev device number for device %s", g_udev_device_get_name (device));
 		return NULL;
 	}
 	/* fun thing: usb device numbers are zero padded, which causes strtol to
 	 * interpret them as octal if you don't specify a base.
 	 */
-	devnum_str = g_udev_device_get_property (G_UDEV_DEVICE (device), "DEVNUM");
+	devnum_str = g_udev_device_get_property (device, "DEVNUM");
 	if (devnum_str == NULL) {
-		rb_debug ("device doesn't have a USB device number");
+		rb_debug ("device %s doesn't have a USB device number", g_udev_device_get_name (device));
 		return NULL;
 	}
 	devnum = strtol (devnum_str, NULL, 10);
 
-	rb_debug ("trying to match device %x (usb device %d) against detected mtp devices",
-		  device_number, devnum);
+	rb_debug ("trying to match device %"G_GINT64_MODIFIER"x (usb device %d) against detected mtp devices",
+		  (guint64)device_number, devnum);
 
 	/* see what devices libmtp can find */
 	if (LIBMTP_Detect_Raw_Devices (&raw_devices, &num_raw_devices) == 0) {
@@ -383,7 +373,8 @@ create_source_device_cb (RBRemovableMediaManager *rmm, GObject *device, RBMtpPlu
 			}
 
 			rb_debug ("device matched, creating a source");
-			source = rb_mtp_source_new (plugin->shell, &raw_devices[i]);
+			source = rb_mtp_source_new (plugin->shell, RB_PLUGIN (plugin), &raw_devices[i]);
+
 			plugin->mtp_sources = g_list_prepend (plugin->mtp_sources, source);
 			g_signal_connect_object (G_OBJECT (source),
 						"deleted", G_CALLBACK (source_deleted_cb),
@@ -428,12 +419,12 @@ rb_mtp_plugin_maybe_add_source (RBMtpPlugin *plugin, const char *udi, LIBMTP_raw
 		rb_debug ("detected MTP device: device number %d (bus location %u)", raw_devices[i].devnum, raw_devices[i].bus_location);
 		if (raw_devices[i].devnum == device_num) {
 			RBSource *source;
+
 			rb_debug ("device matched, creating a source");
-			source = RB_SOURCE (rb_mtp_source_new (plugin->shell, &raw_devices[i], udi));
+			source = RB_SOURCE (rb_mtp_source_new (plugin->shell, RB_PLUGIN (plugin), udi, &raw_devices[i]));
 
 			rb_shell_append_source (plugin->shell, source, NULL);
 			plugin->mtp_sources = g_list_prepend (plugin->mtp_sources, source);
-
 			g_signal_connect_object (G_OBJECT (source),
 						"deleted", G_CALLBACK (source_deleted_cb),
 						plugin, 0);
