@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of base class for play order classes
  *
  *  Copyright (C) 2003 Jeffrey Yasskin <jyasskin@mail.utexas.edu>
  *
@@ -9,6 +7,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -33,10 +39,28 @@
 #include "rb-preferences.h"
 #include "rb-marshal.h"
 
+/**
+ * SECTION:rb-play-order
+ * @short_description: base class for play order implementations
+ *
+ * A play order defines an ordering of the entries from a #RhythmDBQueryModel that
+ * the #RBShellPlayer uses to get the next or previous entry to play.
+ *
+ * Play order methods are invoked when changes are made to the query model, when
+ * the query model is replaced, the playing source is changed, or a new playing
+ * entry is selected.
+ *
+ * The play order must implement methods to check for, retrieve, and move to the
+ * next and previous entries in the play order.  Only the go_next and go_previous
+ * methods actually change anything.
+ *
+ * The play order should also emit the have-next-previous-changed signal to hint that
+ * the availability of either a next or previous entry in the order may have changed.
+ * This information is used to update the sensitivity of the next and previous buttons.
+ */
+
 static void rb_play_order_class_init (RBPlayOrderClass *klass);
 static void rb_play_order_init (RBPlayOrder *porder);
-static GObject *rb_play_order_constructor (GType type, guint n_construct_properties,
-					   GObjectConstructParam *construct_properties);
 static void rb_play_order_dispose (GObject *object);
 static void rb_play_order_finalize (GObject *object);
 static void rb_play_order_set_property (GObject *object,
@@ -101,7 +125,6 @@ rb_play_order_class_init (RBPlayOrderClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	object_class->constructor = rb_play_order_constructor;
 	object_class->dispose = rb_play_order_dispose;
 	object_class->finalize = rb_play_order_finalize;
 	object_class->set_property = rb_play_order_set_property;
@@ -111,6 +134,11 @@ rb_play_order_class_init (RBPlayOrderClass *klass)
 	klass->has_previous = default_has_previous;
 	klass->playing_entry_removed = default_playing_entry_removed;
 
+	/**
+	 * RBPlayOrder:player:
+	 *
+	 * The #RBShellPlayer instance
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_PLAYER,
 					 g_param_spec_object ("player",
@@ -119,6 +147,11 @@ rb_play_order_class_init (RBPlayOrderClass *klass)
 						 	      RB_TYPE_SHELL_PLAYER,
 						 	      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	/**
+	 * RBPlayOrder:playing-entry:
+	 *
+	 * The current playing #RhythmDBEntry
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_PLAYING_ENTRY,
 					 g_param_spec_boxed ("playing-entry",
@@ -127,6 +160,16 @@ rb_play_order_class_init (RBPlayOrderClass *klass)
 							     RHYTHMDB_TYPE_ENTRY,
 						 	     G_PARAM_READWRITE));
 
+	/**
+	 * RBPlayOrder::have-next-previous-changed:
+	 * @porder: the #RBPlayOrder
+	 * @have_next: if %TRUE, the play order has at least one more entry
+	 * @have_previous: if %TRUE, the play order has at least one entry
+	 *    before the current entry
+	 *
+	 * Emitted as a hint to suggest that the sensitivity of next/previous
+	 * buttons may need to be updated.
+	 */
 	rb_play_order_signals[HAVE_NEXT_PREVIOUS_CHANGED] =
 		g_signal_new ("have_next_previous_changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -146,19 +189,6 @@ rb_play_order_init (RBPlayOrder *porder)
 	porder->priv = RB_PLAY_ORDER_GET_PRIVATE (porder);
 }
 
-static GObject *
-rb_play_order_constructor (GType type,
-			   guint n_construct_properties,
-			   GObjectConstructParam *construct_properties)
-{
-	RBPlayOrder *porder;
-
-	porder = RB_PLAY_ORDER (G_OBJECT_CLASS (rb_play_order_parent_class)
-			->constructor (type, n_construct_properties, construct_properties));
-
-	return G_OBJECT (porder);
-}
-
 static void
 rb_play_order_dispose (GObject *object)
 {
@@ -482,7 +512,8 @@ rb_play_order_query_model_changed (RBPlayOrder *porder)
 		g_object_get (porder->priv->source, "query-model", &new_model, NULL);
 
 	if (porder->priv->query_model == new_model) {
-		g_object_unref (new_model);
+		if (new_model != NULL)
+			g_object_unref (new_model);
 		return;
 	}
 
@@ -659,7 +690,7 @@ default_playing_entry_removed (RBPlayOrder *porder,
 		case RB_SOURCE_EOF_STOP:
 		case RB_SOURCE_EOF_RETRY:
 			/* stop playing */
-			rb_shell_player_set_playing_source (player, NULL);
+			rb_shell_player_stop (player);
 			break;
 		case RB_SOURCE_EOF_NEXT:
 			{
