@@ -1,7 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  arch-tag: Implementation of Rhythmbox removable media manager
- *
  *  Copyright (C) 2005 James Livingston  <doclivingston@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -63,6 +61,17 @@
 #include "rb-util.h"
 #include "rb-encoder.h"
 
+#if !GLIB_CHECK_VERSION(2,22,0)
+#define g_mount_unmount_with_operation_finish g_mount_unmount_finish
+#define g_mount_unmount_with_operation(m,f,mo,ca,cb,ud) g_mount_unmount(m,f,ca,cb,ud)
+
+#define g_mount_eject_with_operation_finish g_mount_eject_finish
+#define g_mount_eject_with_operation(m,f,mo,ca,cb,ud) g_mount_eject(m,f,ca,cb,ud)
+
+#define g_volume_eject_with_operation_finish g_volume_eject_finish
+#define g_volume_eject_with_operation(v,f,mo,ca,cb,ud) g_volume_eject(v,f,ca,cb,ud)
+#endif
+
 static void rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass);
 static void rb_removable_media_manager_init (RBRemovableMediaManager *mgr);
 static void rb_removable_media_manager_dispose (GObject *object);
@@ -100,8 +109,6 @@ static void uevent_cb (GUdevClient *client, const char *action, GUdevDevice *dev
 #endif
 
 static void do_transfer (RBRemovableMediaManager *manager);
-static void rb_removable_media_manager_cmd_copy_tracks (GtkAction *action,
-							RBRemovableMediaManager *mgr);
 
 typedef struct
 {
@@ -165,9 +172,6 @@ static GtkActionEntry rb_removable_media_manager_actions [] =
 	{ "RemovableSourceEject", GNOME_MEDIA_EJECT, N_("_Eject"), NULL,
 	  N_("Eject this medium"),
 	  G_CALLBACK (rb_removable_media_manager_cmd_eject_medium) },
-	{ "RemovableSourceCopyAllTracks", GTK_STOCK_CDROM, N_("_Extract to Library"), NULL,
-	  N_("Copy all tracks to the library"),
-	  G_CALLBACK (rb_removable_media_manager_cmd_copy_tracks) },
 	{ "MusicScanMedia", NULL, N_("_Scan Removable Media"), NULL,
 	  N_("Scan for new Removable Media"),
 	  G_CALLBACK (rb_removable_media_manager_cmd_scan_media) },
@@ -326,6 +330,18 @@ rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass)
 	g_type_class_add_private (klass, sizeof (RBRemovableMediaManagerPrivate));
 }
 
+static guint
+uint64_hash (gconstpointer v)
+{
+	return (guint) *(const guint64*)v;
+}
+
+static gboolean
+uint64_equal (gconstpointer a, gconstpointer b)
+{
+	return *((const guint64*)a) == *((const guint64*) b);
+}
+
 static void
 rb_removable_media_manager_init (RBRemovableMediaManager *mgr)
 {
@@ -333,7 +349,7 @@ rb_removable_media_manager_init (RBRemovableMediaManager *mgr)
 
 	priv->volume_mapping = g_hash_table_new (NULL, NULL);
 	priv->mount_mapping = g_hash_table_new (NULL, NULL);
-	priv->device_mapping = g_hash_table_new (g_direct_hash, g_direct_equal);
+	priv->device_mapping = g_hash_table_new_full (uint64_hash, uint64_equal, g_free, NULL);
 	priv->transfer_queue = g_async_queue_new ();
 
 	/*
@@ -562,33 +578,37 @@ mount_removed_cb (GVolumeMonitor *monitor,
 }
 
 #if defined(HAVE_GUDEV)
+
 static void
 uevent_cb (GUdevClient *client, const char *action, GUdevDevice *device, RBRemovableMediaManager *mgr)
 {
 	RBRemovableMediaManagerPrivate *priv = GET_PRIVATE (mgr);
-	GUdevDeviceNumber devnum;
-	devnum = g_udev_device_get_device_number (device);
-	rb_debug ("%s event for %s (%x)", action, g_udev_device_get_sysfs_path (device), devnum);
+	guint64 devnum;
+	devnum = (guint64) g_udev_device_get_device_number (device);
+	rb_debug ("%s event for %s (%"G_GINT64_MODIFIER"x)", action,
+                  g_udev_device_get_sysfs_path (device), devnum);
 
 	if (g_str_equal (action, "add")) {
 		RBSource *source = NULL;
 
 		/* probably need to filter out devices related to things we've already seen.. */
-		if (g_hash_table_lookup (priv->device_mapping, GINT_TO_POINTER (devnum)) != NULL) {
+		if (g_hash_table_lookup (priv->device_mapping, &devnum) != NULL) {
 			rb_debug ("already have a source for this device");
 			return;
 		}
 
 		g_signal_emit (mgr, rb_removable_media_manager_signals[CREATE_SOURCE_DEVICE], 0, device, &source);
 		if (source != NULL) {
+			guint64 *key = g_new0 (guint64, 1);
 			rb_debug ("created a source for this device");
-			g_hash_table_insert (priv->device_mapping, GINT_TO_POINTER (devnum), source);
+			key[0] = devnum;
+			g_hash_table_insert (priv->device_mapping, key, source);
 			rb_removable_media_manager_append_media_source (mgr, source);
 		}
 	} else if (g_str_equal (action, "remove")) {
 		RBSource *source;
 
-		source = g_hash_table_lookup (priv->device_mapping, GINT_TO_POINTER (devnum));
+		source = g_hash_table_lookup (priv->device_mapping, &devnum);
 		if (source) {
 			rb_debug ("removing the source created for this device");
 			rb_source_delete_thyself (source);
@@ -799,7 +819,6 @@ rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr,
 					  GtkUIManager *uimanager)
 {
 	RBRemovableMediaManagerPrivate *priv = GET_PRIVATE (mgr);
-	GtkAction *action;
 
 	if (priv->uimanager != NULL) {
 		if (priv->actiongroup != NULL) {
@@ -829,13 +848,6 @@ rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr,
 	gtk_ui_manager_insert_action_group (priv->uimanager,
 					    priv->actiongroup,
 					    0);
-
-	action = gtk_action_group_get_action (priv->actiongroup,
-					      "RemovableSourceCopyAllTracks");
-	/* Translators: this is the toolbar button label
-	   for Copy to Library action. */
-	g_object_set (G_OBJECT (action), "short-label", _("Extract"), NULL);
-
 }
 
 static void
@@ -849,7 +861,7 @@ rb_removable_media_manager_eject_cb (GObject *object,
 		GVolume *volume = G_VOLUME (object);
 
 		rb_debug ("finishing ejection of volume");
-		g_volume_eject_finish (volume, result, &error);
+		g_volume_eject_with_operation_finish (volume, result, &error);
 		if (error == NULL) {
 			rb_removable_media_manager_remove_volume (mgr, volume);
 		}
@@ -857,7 +869,7 @@ rb_removable_media_manager_eject_cb (GObject *object,
 		GMount *mount = G_MOUNT (object);
 
 		rb_debug ("finishing ejection of mount");
-		g_mount_eject_finish (mount, result, &error);
+		g_mount_eject_with_operation_finish (mount, result, &error);
 		if (error == NULL) {
 			rb_removable_media_manager_remove_mount (mgr, mount);
 		}
@@ -883,7 +895,7 @@ rb_removable_media_manager_unmount_cb (GObject *object,
 	GError *error = NULL;
 
 	rb_debug ("finishing unmount of mount");
-	g_mount_unmount_finish (mount, result, &error);
+	g_mount_unmount_with_operation_finish (mount, result, &error);
 	if (error != NULL) {
 		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_FAILED_HANDLED)) {
 			rb_error_dialog (NULL, _("Unable to unmount"), "%s", error->message);
@@ -942,11 +954,12 @@ rb_removable_media_manager_cmd_eject_medium (GtkAction *action, RBRemovableMedia
 	if (volume != NULL) {
 		if (g_volume_can_eject (volume)) {
 			rb_debug ("ejecting volume");
-			g_volume_eject (volume,
-					G_MOUNT_UNMOUNT_NONE,
-					NULL,
-					(GAsyncReadyCallback) rb_removable_media_manager_eject_cb,
-					g_object_ref (mgr));
+			g_volume_eject_with_operation (volume,
+						       G_MOUNT_UNMOUNT_NONE,
+						       NULL,
+						       NULL,
+						       (GAsyncReadyCallback) rb_removable_media_manager_eject_cb,
+						       g_object_ref (mgr));
 		} else {
 			/* this should never happen; the eject command will be
 			 * insensitive if the selected source cannot be ejected.
@@ -961,18 +974,20 @@ rb_removable_media_manager_cmd_eject_medium (GtkAction *action, RBRemovableMedia
 	if (mount != NULL) {
 		if (g_mount_can_eject (mount)) {
 			rb_debug ("ejecting mount");
-			g_mount_eject (mount,
-				       G_MOUNT_UNMOUNT_NONE,
-				       NULL,
-				       (GAsyncReadyCallback) rb_removable_media_manager_eject_cb,
-				       g_object_ref (mgr));
+			g_mount_eject_with_operation (mount,
+						      G_MOUNT_UNMOUNT_NONE,
+						      NULL,
+						      NULL,
+						      (GAsyncReadyCallback) rb_removable_media_manager_eject_cb,
+						      g_object_ref (mgr));
 		} else if (g_mount_can_unmount (mount)) {
 			rb_debug ("unmounting mount");
-			g_mount_unmount (mount,
-					 G_MOUNT_UNMOUNT_NONE,
-					 NULL,
-					 (GAsyncReadyCallback) rb_removable_media_manager_unmount_cb,
-					 g_object_ref (mgr));
+			g_mount_unmount_with_operation (mount,
+							G_MOUNT_UNMOUNT_NONE,
+							NULL,
+							NULL,
+							(GAsyncReadyCallback) rb_removable_media_manager_unmount_cb,
+							g_object_ref (mgr));
 		} else {
 			/* this should never happen; the eject command will be
 			 * insensitive if the selected source cannot be ejected.
@@ -1106,12 +1121,12 @@ progress_cb (RBEncoder *encoder, double fraction, TransferData *data)
 }
 
 static void
-completed_cb (RBEncoder *encoder, TransferData *data)
+completed_cb (RBEncoder *encoder, guint64 dest_size, TransferData *data)
 {
 	RBRemovableMediaManagerPrivate *priv = GET_PRIVATE (data->manager);
 
 	rb_debug ("completed transferring track to %s", data->dest);
-	(data->callback) (data->entry, data->dest, data->error, data->userdata);
+	(data->callback) (data->entry, data->dest, dest_size, data->error, data->userdata);
 
 	priv->transfer_running = FALSE;
 	priv->transfer_done++;
@@ -1210,35 +1225,3 @@ rb_removable_media_manager_queue_transfer (RBRemovableMediaManager *manager,
 	do_transfer (manager);
 }
 
-static gboolean
-copy_entry (RhythmDBQueryModel *model,
-	    GtkTreePath *path,
-	    GtkTreeIter *iter,
-	    GList **list)
-{
-	GList *l;
-	l = g_list_append (*list, rhythmdb_query_model_iter_to_entry (model, iter));
-	*list = l;
-	return FALSE;
-}
-
-static void
-rb_removable_media_manager_cmd_copy_tracks (GtkAction *action, RBRemovableMediaManager *mgr)
-{
-	RBRemovableMediaManagerPrivate *priv = GET_PRIVATE (mgr);
-	RBRemovableMediaSource *source;
-	RBLibrarySource *library;
-	RhythmDBQueryModel *model;
-	GList *list = NULL;
-
-	source = RB_REMOVABLE_MEDIA_SOURCE (priv->selected_source);
-	g_object_get (source, "query-model", &model, NULL);
-	g_object_get (priv->shell, "library-source", &library, NULL);
-
-	gtk_tree_model_foreach (GTK_TREE_MODEL (model), (GtkTreeModelForeachFunc)copy_entry, &list);
-	rb_source_paste (RB_SOURCE (library), list);
-	g_list_free (list);
-
-	g_object_unref (model);
-	g_object_unref (library);
-}
