@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Header for object implementing main playback logic
- *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
  *
@@ -9,6 +7,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -20,9 +26,7 @@
  *
  */
 
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkuimanager.h>
-#include <gtk/gtkactiongroup.h>
+#include <gtk/gtk.h>
 
 #include "rb-source.h"
 #include "rhythmdb.h"
@@ -54,22 +58,24 @@ GType rb_shell_player_error_get_type (void);
 #define RB_SHELL_PLAYER_ERROR rb_shell_player_error_quark ()
 
 GQuark rb_shell_player_error_quark (void);
-
+typedef struct _RBShellPlayer RBShellPlayer;
+typedef struct _RBShellPlayerClass RBShellPlayerClass;
 typedef struct RBShellPlayerPrivate RBShellPlayerPrivate;
 
-typedef struct
+struct _RBShellPlayer
 {
 	GtkHBox parent;
 
 	RBShellPlayerPrivate *priv;
-} RBShellPlayer;
+};
 
-typedef struct
+struct _RBShellPlayerClass
 {
 	GtkHBoxClass parent_class;
 
 	void (*window_title_changed) (RBShellPlayer *player, const char *window_title);
 	void (*elapsed_changed) (RBShellPlayer *player, guint elapsed);
+	void (*elapsed_nano_changed) (RBShellPlayer *player, gint64 elapsed);
 	void (*playing_changed) (RBShellPlayer *player, gboolean playing);
 	void (*playing_source_changed) (RBShellPlayer *player, RBSource *source);
 	void (*playing_uri_changed) (RBShellPlayer *player, const char *uri);
@@ -79,7 +85,7 @@ typedef struct
 					       const char *property,
 					       GValue *old,
 					       GValue *newValue);
-} RBShellPlayerClass;
+};
 
 GType			rb_shell_player_get_type   (void);
 
@@ -87,13 +93,13 @@ RBShellPlayer *		rb_shell_player_new		(RhythmDB *db,
 							 GtkUIManager *mgr,
 							 GtkActionGroup *actiongroup);
 
-void			rb_shell_player_set_selected_source	(RBShellPlayer *shell_player,
-								 RBSource *player);
+void			rb_shell_player_set_selected_source	(RBShellPlayer *player,
+								 RBSource *source);
 void			rb_shell_player_set_playing_source (RBShellPlayer *player,
 							    RBSource *source);
 
-RBSource *		rb_shell_player_get_playing_source (RBShellPlayer *shell_player);
-RBSource *		rb_shell_player_get_active_source (RBShellPlayer *shell_player);
+RBSource *		rb_shell_player_get_playing_source (RBShellPlayer *player);
+RBSource *		rb_shell_player_get_active_source (RBShellPlayer *player);
 
 void			rb_shell_player_jump_to_current (RBShellPlayer *player);
 
@@ -117,11 +123,11 @@ gboolean		rb_shell_player_set_playing_time(RBShellPlayer *player,
 void			rb_shell_player_seek		(RBShellPlayer *player, long offset);
 long			rb_shell_player_get_playing_song_duration (RBShellPlayer *player);
 
-gboolean		rb_shell_player_get_playing	(RBShellPlayer *shell_player,
+gboolean		rb_shell_player_get_playing	(RBShellPlayer *player,
 							 gboolean *playing,
 							 GError **error);
 
-gboolean		rb_shell_player_get_playing_path(RBShellPlayer *shell_player,
+gboolean		rb_shell_player_get_playing_path(RBShellPlayer *player,
 							 const gchar **path,
 							 GError **error);
 
@@ -145,7 +151,7 @@ gboolean		rb_shell_player_get_volume	(RBShellPlayer *player,
 							 GError **error);
 
 gboolean		rb_shell_player_set_volume_relative (RBShellPlayer *player,
-							 gdouble volume,
+							 gdouble delta,
 							 GError **error);
 
 gboolean		rb_shell_player_set_mute	(RBShellPlayer *player,
