@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of local file source object
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
@@ -70,9 +68,7 @@
 
 static void rb_library_source_class_init (RBLibrarySourceClass *klass);
 static void rb_library_source_init (RBLibrarySource *source);
-static GObject *rb_library_source_constructor (GType type,
-					       guint n_construct_properties,
-					       GObjectConstructParam *construct_properties);
+static void rb_library_source_constructed (GObject *object);
 static void rb_library_source_dispose (GObject *object);
 static void rb_library_source_finalize (GObject *object);
 
@@ -86,6 +82,7 @@ static gboolean impl_can_paste (RBSource *asource);
 static void impl_paste (RBSource *source, GList *entries);
 static guint impl_want_uri (RBSource *source, const char *uri);
 static gboolean impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre);
+static void impl_get_status (RBSource *source, char **text, char **progress_text, float *progress);
 
 static void rb_library_source_ui_prefs_sync (RBLibrarySource *source);
 static void rb_library_source_preferences_sync (RBLibrarySource *source);
@@ -121,6 +118,7 @@ static void rb_library_source_filename_changed_cb (GtkComboBox *box,
 static void rb_library_source_format_changed_cb (GtkWidget *widget,
 						 RBLibrarySource *source);
 static void layout_example_label_update (RBLibrarySource *source);
+static RhythmDBImportJob *maybe_create_import_job (RBLibrarySource *source);
 
 #define CONF_UI_LIBRARY_DIR CONF_PREFIX "/ui/library"
 #define CONF_STATE_LIBRARY_DIR CONF_PREFIX "/state/library"
@@ -170,6 +168,9 @@ struct RBLibrarySourcePrivate
 	GtkWidget *preferred_format_menu;
 	GtkWidget *layout_example_label;
 
+	GList *import_jobs;
+	guint start_import_job_id;
+
 	guint library_location_notify_id;
 	guint ui_dir_notify_id;
 	guint layout_path_notify_id;
@@ -188,7 +189,7 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 
 	object_class->dispose = rb_library_source_dispose;
 	object_class->finalize = rb_library_source_finalize;
-	object_class->constructor = rb_library_source_constructor;
+	object_class->constructed = rb_library_source_constructed;
 
 	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_get_config_widget = impl_get_config_widget;
@@ -199,6 +200,7 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 	source_class->impl_paste = impl_paste;
 	source_class->impl_want_uri = impl_want_uri;
 	source_class->impl_add_uri = impl_add_uri;
+	source_class->impl_get_status = impl_get_status;
 
 	browser_source_class->impl_get_paned_key = impl_get_paned_key;
 	browser_source_class->impl_has_drop_support = (RBBrowserSourceFeatureFunc) rb_true_function;
@@ -250,6 +252,21 @@ rb_library_source_dispose (GObject *object)
 		source->priv->layout_filename_notify_id = 0;
 	}
 
+	if (source->priv->import_jobs != NULL) {
+		GList *t;
+		if (source->priv->start_import_job_id != 0) {
+			g_source_remove (source->priv->start_import_job_id);
+			source->priv->start_import_job_id = 0;
+		}
+		for (t = source->priv->import_jobs; t != NULL; t = t->next) {
+			RhythmDBImportJob *job = RHYTHMDB_IMPORT_JOB (t->data);
+			rhythmdb_import_job_cancel (job);
+			g_object_unref (job);
+		}
+		g_list_free (source->priv->import_jobs);
+		source->priv->import_jobs = NULL;
+	}
+
 	G_OBJECT_CLASS (rb_library_source_parent_class)->dispose (object);
 }
 
@@ -287,18 +304,16 @@ db_load_complete_cb (RhythmDB *db, RBLibrarySource *source)
 	g_object_set (source, "populate", TRUE, NULL);
 }
 
-static GObject *
-rb_library_source_constructor (GType type,
-			       guint n_construct_properties,
-			       GObjectConstructParam *construct_properties)
+static void
+rb_library_source_constructed (GObject *object)
 {
 	RBLibrarySource *source;
 	RBShell *shell;
 	RBEntryView *songs;
 	GSList *list;
 
-	source = RB_LIBRARY_SOURCE (G_OBJECT_CLASS (rb_library_source_parent_class)
-			->constructor (type, n_construct_properties, construct_properties));
+	RB_CHAIN_GOBJECT_METHOD (rb_library_source_parent_class, constructed, object);
+	source = RB_LIBRARY_SOURCE (object);
 
 	g_object_get (source, "shell", &shell, NULL);
 	g_object_get (shell, "db", &source->priv->db, NULL);
@@ -359,8 +374,6 @@ rb_library_source_constructor (GType type,
 	g_idle_add ((GSourceFunc)add_child_sources_idle, source);
 
 	g_object_unref (shell);
-
-	return G_OBJECT (source);
 }
 
 /**
@@ -689,8 +702,8 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 	gboolean is_id;
 
 	rb_debug ("parsing uri list");
-	list = rb_uri_list_parse ((const char *) data->data);
-	is_id = (data->type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE));
+	list = rb_uri_list_parse ((const char *) gtk_selection_data_get_data (data));
+	is_id = (gtk_selection_data_get_data_type (data) == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE));
 
 	for (i = list; i != NULL; i = g_list_next (i)) {
 		if (i->data != NULL) {
@@ -700,8 +713,10 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 			entry = rhythmdb_entry_lookup_from_string (source->priv->db, uri, is_id);
 
 			if (entry == NULL) {
+				RhythmDBImportJob *job;
 				/* add to the library */
-				rhythmdb_add_uri (source->priv->db, uri);
+				job = maybe_create_import_job (source);
+				rhythmdb_import_job_add_uri (job, uri);
 			} else {
 				/* add to list of entries to copy */
 				entries = g_list_prepend (entries, entry);
@@ -827,17 +842,30 @@ sanitize_pattern (const char *pat)
  * %tS -- track artist sortname (lowercase)
  */
 static char *
-filepath_parse_pattern (const char *pattern,
+filepath_parse_pattern (RhythmDB *db,
+			const char *pattern,
 			RhythmDBEntry *entry)
 {
 	/* p is the pattern iterator, i is a general purpose iterator */
 	const char *p;
 	char *temp;
 	GString *s;
+	GValue *value;
+	RBRefString *albumartist;
 
 	if (pattern == NULL || pattern[0] == 0)
 		return g_strdup (" ");
 
+	/* request album artist (this is sort of temporary) */
+	value = rhythmdb_entry_request_extra_metadata (db, entry, RHYTHMDB_PROP_ALBUM_ARTIST);
+	if (value != NULL) {
+		albumartist = rb_refstring_new (g_value_get_string (value));
+		g_value_unset (value);
+		g_free (value);
+	} else {
+		albumartist = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_ARTIST);
+	}
+
 	s = g_string_new (NULL);
 
 	p = pattern;
@@ -872,10 +900,10 @@ filepath_parse_pattern (const char *pattern,
 				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM_FOLDED));
 				break;
 			case 'a':
-				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST));
+				string = sanitize_path (rb_refstring_get (albumartist));
 				break;
 			case 'A':
-				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_FOLDED));
+				string = sanitize_path (rb_refstring_get_folded (albumartist));
 				break;
 			case 's':
 				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME));
@@ -943,6 +971,7 @@ filepath_parse_pattern (const char *pattern,
 
 	temp = s->str;
 	g_string_free (s, FALSE);
+	rb_refstring_unref (albumartist);
 	return temp;
 }
 
@@ -980,8 +1009,8 @@ layout_example_label_update (RBLibrarySource *source)
 	sample_entry = rhythmdb_entry_example_new (source->priv->db, entry_type, NULL);
 	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
-	file_value = filepath_parse_pattern (file_pattern, sample_entry);
-	path_value = filepath_parse_pattern (path_pattern, sample_entry);
+	file_value = filepath_parse_pattern (source->priv->db, file_pattern, sample_entry);
+	path_value = filepath_parse_pattern (source->priv->db, path_pattern, sample_entry);
 	rhythmdb_entry_unref (sample_entry);
 
 	example = g_build_filename (G_DIR_SEPARATOR_S, path_value, file_value, NULL);
@@ -1012,6 +1041,7 @@ rb_library_source_layout_path_changed (GConfClient *client,
 	char *value;
 	int i = 0;
 
+	g_return_if_fail (entry != NULL);
 	g_return_if_fail (strcmp (entry->key, CONF_LIBRARY_LAYOUT_PATH) == 0);
 
 	rb_debug ("layout path changed");
@@ -1043,6 +1073,7 @@ rb_library_source_layout_filename_changed (GConfClient *client,
 	char *value;
 	int i = 0;
 
+	g_return_if_fail (entry != NULL);
 	g_return_if_fail (strcmp (entry->key, CONF_LIBRARY_LAYOUT_FILENAME) == 0);
 
 	rb_debug ("layout filename changed");
@@ -1106,7 +1137,7 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	g_free (layout_filename);
 	layout_filename = tmp;
 
-	realpath = filepath_parse_pattern (layout_path, entry);
+	realpath = filepath_parse_pattern (source->priv->db, layout_path, entry);
 
 	library_location = g_file_new_for_uri ((const char *)list->data);
 	dir = g_file_resolve_relative_path (library_location, realpath);
@@ -1140,7 +1171,7 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 			*tmp = '\0';
 	}
 
-	realfile = filepath_parse_pattern (layout_filename, entry);
+	realfile = filepath_parse_pattern (source->priv->db, layout_filename, entry);
 	if (extension) {
 		filename = g_strdup_printf ("%s.%s", realfile, extension);
 		g_free (realfile);
@@ -1190,9 +1221,17 @@ impl_can_paste (RBSource *asource)
 }
 
 static void
-completed_cb (RhythmDBEntry *entry, const char *dest, guint64 dest_size, RBLibrarySource *source)
+completed_cb (RhythmDBEntry *entry, const char *dest, guint64 dest_size, GError *error, RBLibrarySource *source)
 {
-	rhythmdb_add_uri (source->priv->db, dest);
+	if (error == NULL) {
+		rhythmdb_add_uri (source->priv->db, dest);
+	} else {
+		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+			rb_debug ("not displaying 'file exists' error for %s", dest);
+		} else {
+			rb_error_dialog (NULL, _("Error transferring track"), "%s", error->message);
+		}
+	}
 }
 
 static void
@@ -1272,13 +1311,83 @@ impl_want_uri (RBSource *source, const char *uri)
 	return 0;
 }
 
+static void
+import_job_status_changed_cb (RhythmDBImportJob *job, int total, int imported, RBLibrarySource *source)
+{
+	RhythmDBImportJob *head = RHYTHMDB_IMPORT_JOB (source->priv->import_jobs->data);
+	if (job == head) {		/* it was inevitable */
+		rb_source_notify_status_changed (RB_SOURCE (source));
+	}
+}
+
+static void
+import_job_complete_cb (RhythmDBImportJob *job, int total, RBLibrarySource *source)
+{
+	rb_debug ("import job complete");
+
+	/* maybe show a notification here? */
+
+	source->priv->import_jobs = g_list_remove (source->priv->import_jobs, job);
+	g_object_unref (job);
+}
+
+static gboolean
+start_import_job (RBLibrarySource *source)
+{
+	RhythmDBImportJob *job;
+	source->priv->start_import_job_id = 0;
+
+	rb_debug ("starting import job");
+	job = RHYTHMDB_IMPORT_JOB (source->priv->import_jobs->data);
+
+	rhythmdb_import_job_start (job);
+
+	return FALSE;
+}
+
+static RhythmDBImportJob *
+maybe_create_import_job (RBLibrarySource *source)
+{
+	RhythmDBImportJob *job;
+	if (source->priv->import_jobs == NULL || source->priv->start_import_job_id == 0) {
+		rb_debug ("creating new import job");
+		job = rhythmdb_import_job_new (source->priv->db,
+					       RHYTHMDB_ENTRY_TYPE_SONG,
+					       RHYTHMDB_ENTRY_TYPE_IGNORE,
+					       RHYTHMDB_ENTRY_TYPE_IMPORT_ERROR);
+		g_signal_connect_object (job,
+					 "status-changed",
+					 G_CALLBACK (import_job_status_changed_cb),
+					 source, 0);
+		g_signal_connect_object (job,
+					 "complete",
+					 G_CALLBACK (import_job_complete_cb),
+					 source, 0);
+		source->priv->import_jobs = g_list_prepend (source->priv->import_jobs, job);
+	} else {
+		rb_debug ("using existing unstarted import job");
+		job = RHYTHMDB_IMPORT_JOB (source->priv->import_jobs->data);
+	}
+
+	/* allow some time for more URIs to be added if we're importing a bunch of things */
+	if (source->priv->start_import_job_id != 0) {
+		g_source_remove (source->priv->start_import_job_id);
+	}
+	source->priv->start_import_job_id = g_timeout_add (250, (GSourceFunc) start_import_job, source);
+
+	return job;
+}
+
 static gboolean
 impl_add_uri (RBSource *asource, const char *uri, const char *title, const char *genre)
 {
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
-	/* FIXME should be synchronous */
+	RhythmDBImportJob *job;
+
+	job = maybe_create_import_job (source);
+
 	rb_debug ("adding uri %s to library", uri);
-	rhythmdb_add_uri (source->priv->db, uri);
+	rhythmdb_import_job_add_uri (job, uri);
 	return TRUE;
 }
 
@@ -1349,3 +1458,14 @@ rb_library_source_sync_child_sources (RBLibrarySource *source)
 	rb_slist_deep_free (list);
 }
 
+static void
+impl_get_status (RBSource *source, char **text, char **progress_text, float *progress)
+{
+	RB_SOURCE_CLASS (rb_library_source_parent_class)->impl_get_status (source, text, progress_text, progress);
+	RBLibrarySource *lsource = RB_LIBRARY_SOURCE (source);
+
+	if (lsource->priv->import_jobs != NULL) {
+		RhythmDBImportJob *job = RHYTHMDB_IMPORT_JOB (lsource->priv->import_jobs->data);
+		_rb_source_set_import_status (source, job, progress_text, progress);
+	}
+}
