@@ -1,7 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  arch-tag: Implementation of playlist source object
- *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
  *
@@ -120,13 +118,25 @@ static void rb_playlist_source_songs_sort_order_changed_cb (RBEntryView *view,
 						RBStaticPlaylistSource *source);
 static char *rb_playlist_source_make_sorting_key (RBPlaylistSource *source);
 
+static void remove_from_playlist_cmd (GtkAction *action, RBSource *source);
+static char *impl_get_delete_action (RBSource *source);
+
 #define CONF_STATE_SORTING_PREFIX CONF_PREFIX "/state/sorting/"
 #define PLAYLIST_SOURCE_SONGS_POPUP_PATH "/PlaylistViewPopup"
 #define PLAYLIST_SOURCE_POPUP_PATH "/PlaylistSourcePopup"
 
+static GtkActionEntry rb_playlist_source_actions [] =
+{
+	{ "RemoveFromPlaylist", GTK_STOCK_REMOVE, N_("Remove From Playlist"), NULL,
+	  N_("Remove each selected song from the playlist"),
+	  G_CALLBACK (remove_from_playlist_cmd) },
+};
+
+
 struct RBPlaylistSourcePrivate
 {
 	RhythmDB *db;
+	GtkActionGroup *action_group;
 
 	GHashTable *entries;
 
@@ -181,6 +191,7 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 	source_class->impl_song_properties = impl_song_properties;
 	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_show_popup = impl_show_popup;
+	source_class->impl_get_delete_action = impl_get_delete_action;
 
 	klass->impl_show_entry_view_popup = default_show_entry_view_popup;
 	klass->impl_mark_dirty = default_mark_dirty;
@@ -287,6 +298,16 @@ rb_playlist_source_constructed (GObject *object)
 	shell_player = rb_shell_get_player (shell);
 	rb_playlist_source_set_db (source, db);
 	g_object_unref (db);
+
+	source->priv->action_group = _rb_source_register_action_group (RB_SOURCE (source),
+								       "PlaylistActions",
+								       NULL, 0,
+								       shell);
+	_rb_action_group_add_source_actions (source->priv->action_group,
+					     G_OBJECT (shell),
+					     rb_playlist_source_actions,
+					     G_N_ELEMENTS (rb_playlist_source_actions));
+
 	g_object_unref (shell);
 
 	source->priv->entries = g_hash_table_new_full (rb_refstring_hash, rb_refstring_equal,
@@ -546,6 +567,52 @@ rb_playlist_source_drop_cb (GtkWidget *widget,
 	gtk_drag_finish (context, TRUE, FALSE, time);
 }
 
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,29,1)
+
+static void
+set_field_from_property (TotemPlPlaylist *playlist,
+			 TotemPlPlaylistIter *iter,
+			 RhythmDBEntry *entry,
+			 RhythmDBPropType property,
+			 const char *field)
+{
+	const char *value;
+
+	value = rhythmdb_entry_get_string (entry, property);
+	if (value != NULL) {
+		totem_pl_playlist_set (playlist, iter, field, value, NULL);
+	}
+}
+
+static gboolean
+playlist_iter_foreach (GtkTreeModel *model,
+		       GtkTreePath *path,
+		       GtkTreeIter *iter,
+		       TotemPlPlaylist *playlist)
+{
+	TotemPlPlaylistIter pl_iter;
+	RhythmDBEntry *entry;
+
+	gtk_tree_model_get (model, iter, 0, &entry, -1);
+	if (entry == NULL) {
+		return FALSE;
+	}
+
+	totem_pl_playlist_append (playlist, &pl_iter);
+	set_field_from_property (playlist, &pl_iter, entry, RHYTHMDB_PROP_LOCATION, TOTEM_PL_PARSER_FIELD_URI);
+	set_field_from_property (playlist, &pl_iter, entry, RHYTHMDB_PROP_ARTIST, TOTEM_PL_PARSER_FIELD_AUTHOR);
+	set_field_from_property (playlist, &pl_iter, entry, RHYTHMDB_PROP_GENRE, TOTEM_PL_PARSER_FIELD_GENRE);
+	set_field_from_property (playlist, &pl_iter, entry, RHYTHMDB_PROP_ALBUM, TOTEM_PL_PARSER_FIELD_ALBUM);
+	set_field_from_property (playlist, &pl_iter, entry, RHYTHMDB_PROP_TITLE, TOTEM_PL_PARSER_FIELD_TITLE);
+
+	/* could possibly set duration, file size.. ? */
+
+	return FALSE;
+}
+
+
+#else
+
 static void
 playlist_iter_func (GtkTreeModel *model,
 		    GtkTreeIter *iter,
@@ -565,7 +632,7 @@ playlist_iter_func (GtkTreeModel *model,
 		*title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
 	}
 	if (custom_title != NULL) {
-		*custom_title = FALSE;
+		*custom_title = TRUE;
 	}
 
 	if (entry != NULL) {
@@ -573,6 +640,8 @@ playlist_iter_func (GtkTreeModel *model,
 	}
 }
 
+#endif
+
 /**
  * rb_playlist_source_save_playlist:
  * @source: a #RBPlaylistSource
@@ -587,15 +656,19 @@ rb_playlist_source_save_playlist (RBPlaylistSource *source,
 				  const char *uri,
 				  RBPlaylistExportType export_type)
 {
-	TotemPlParser *playlist;
+	TotemPlParser *pl;
 	GError *error = NULL;
 	char *name;
 	gint totem_format;
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,29,1)
+	TotemPlPlaylist *playlist;
+	GFile *file;
+#endif
 
 	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
 
 	rb_debug ("saving playlist");
-	playlist = totem_pl_parser_new ();
+	pl = totem_pl_parser_new ();
 
 	g_object_get (source, "name", &name, NULL);
 
@@ -612,11 +685,23 @@ rb_playlist_source_save_playlist (RBPlaylistSource *source,
 		break;
 	}
 
-	totem_pl_parser_write_with_title (playlist, GTK_TREE_MODEL (source->priv->model),
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,29,1)
+	file = g_file_new_for_uri (uri);
+	playlist = totem_pl_playlist_new ();
+
+	gtk_tree_model_foreach (GTK_TREE_MODEL (source->priv->model),
+				(GtkTreeModelForeachFunc)playlist_iter_foreach,
+				playlist);
+	totem_pl_parser_save (pl, playlist, file, name, totem_format, &error);
+	g_object_unref (playlist);
+	g_object_unref (file);
+#else
+	totem_pl_parser_write_with_title (pl, GTK_TREE_MODEL (source->priv->model),
 					  playlist_iter_func, uri, name,
 					  totem_format,
 					  NULL, &error);
-	g_object_unref (playlist);
+#endif
+	g_object_unref (pl);
 	g_free (name);
 	if (error != NULL) {
 		rb_error_dialog (NULL, _("Couldn't save playlist"),
@@ -1068,3 +1153,14 @@ rb_playlist_source_make_sorting_key (RBPlaylistSource *source)
 	return sorting_key;
 }
 
+static void
+remove_from_playlist_cmd (GtkAction *action, RBSource *source)
+{
+	rb_source_delete (source);
+}
+
+static char *
+impl_get_delete_action (RBSource *source)
+{
+	return g_strdup ("RemoveFromPlaylist");
+}
