@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of removable media source object (based of the ipod source)
  *
  *  Copyright (C) 2004 Christophe Fergeau  <teuf@gnome.org>
  *  Copyright (C) 2005 James Livingston  <doclivingston@gmail.com>
@@ -66,6 +64,7 @@ static gboolean impl_should_paste (RBRemovableMediaSource *source,
 				   RhythmDBEntry *entry);
 static guint impl_want_uri (RBSource *source, const char *uri);
 static gboolean impl_uri_is_source (RBSource *source, const char *uri);
+static char *impl_get_delete_action (RBSource *source);
 
 typedef struct
 {
@@ -108,6 +107,7 @@ rb_removable_media_source_class_init (RBRemovableMediaSourceClass *klass)
 	source_class->impl_show_popup = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_want_uri = impl_want_uri;
 	source_class->impl_uri_is_source = impl_uri_is_source;
+	source_class->impl_get_delete_action = impl_get_delete_action;
 
 	browser_source_class->impl_get_paned_key = NULL;
 	browser_source_class->impl_has_drop_support = (RBBrowserSourceFeatureFunc) rb_false_function;
@@ -306,10 +306,10 @@ struct _TrackAddedData {
 };
 
 static void
-_track_added_cb (RhythmDBEntry *entry, const char *uri, GError *error, struct _TrackAddedData *data)
+_track_added_cb (RhythmDBEntry *entry, const char *uri, guint64 dest_size, GError *error, struct _TrackAddedData *data)
 {
 	if (error == NULL) {
-		rb_removable_media_source_track_added (data->source, entry, uri, data->mimetype);
+		rb_removable_media_source_track_added (data->source, entry, uri, dest_size, data->mimetype);
 	} else {
 		rb_removable_media_source_track_add_error (data->source, entry, uri, error);
 	}
@@ -521,7 +521,7 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 	char *type;
 
 	entries = NULL;
-	type = gdk_atom_name (data->type);
+	type = gdk_atom_name (gtk_selection_data_get_data_type (data));
         db = get_db_for_source (asource);
 
 	if (strcmp (type, "text/uri-list") == 0) {
@@ -529,7 +529,7 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 		GList *i;
 
 		rb_debug ("parsing uri list");
-		list = rb_uri_list_parse ((const char *) data->data);
+		list = rb_uri_list_parse ((const char *) gtk_selection_data_get_data (data));
 
 		for (i = list; i != NULL; i = g_list_next (i)) {
 			char *uri;
@@ -556,7 +556,7 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 		char **i;
 
 		rb_debug ("parsing entry ids");
-		list = g_strsplit ((const char*)data->data, "\n", -1);
+		list = g_strsplit ((const char*) gtk_selection_data_get_data (data), "\n", -1);
 		for (i = list; *i != NULL; i++) {
 			RhythmDBEntry *entry;
 			gulong id;
@@ -625,6 +625,33 @@ rb_removable_media_source_get_mime_types (RBRemovableMediaSource *source)
 		return NULL;
 }
 
+GList *
+rb_removable_media_source_get_format_descriptions (RBRemovableMediaSource *source)
+{
+	GList *mime;
+	GList *desc = NULL;
+	GList *t;
+
+	mime = rb_removable_media_source_get_mime_types (source);
+	for (t = mime; t != NULL; t = t->next) {
+		const char *mimetype;
+		char *content_type;
+
+		mimetype = t->data;
+		content_type = g_content_type_from_mime_type (mimetype);
+		if (content_type != NULL) {
+			char *description;
+			description = g_content_type_get_description (content_type);
+			desc = g_list_append (desc, description);
+		} else {
+			desc = g_list_append (desc, g_strdup (mimetype));
+		}
+	}
+
+	rb_list_deep_free (mime);
+	return desc;
+}
+
 gboolean
 rb_removable_media_source_should_paste_no_duplicate (RBRemovableMediaSource *source,
 						     RhythmDBEntry *entry)
@@ -635,6 +662,7 @@ rb_removable_media_source_should_paste_no_duplicate (RBRemovableMediaSource *sou
 	const char *title;
 	const char *album;
 	const char *artist;
+	gulong track_number;
 	GtkTreeModel *query_model;
 	GtkTreeIter iter;
 	gboolean no_match;
@@ -652,6 +680,7 @@ rb_removable_media_source_should_paste_no_duplicate (RBRemovableMediaSource *sou
 	title = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE);
 	album = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM);
 	artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
+	track_number = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER);
 	rhythmdb_do_full_query (db, RHYTHMDB_QUERY_RESULTS (query_model),
 				RHYTHMDB_QUERY_PROP_EQUALS,
 				RHYTHMDB_PROP_TYPE, entry_type,
@@ -661,6 +690,8 @@ rb_removable_media_source_should_paste_no_duplicate (RBRemovableMediaSource *sou
 				RHYTHMDB_PROP_ALBUM, album,
 				RHYTHMDB_QUERY_PROP_EQUALS,
 				RHYTHMDB_PROP_TITLE, title,
+				RHYTHMDB_QUERY_PROP_EQUALS,
+				RHYTHMDB_PROP_TRACK_NUMBER, track_number,
 				RHYTHMDB_QUERY_END);
 
 	no_match = (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (query_model),
@@ -669,7 +700,7 @@ rb_removable_media_source_should_paste_no_duplicate (RBRemovableMediaSource *sou
 	g_object_unref(query_model);
 	g_object_unref (db);
 	if (no_match == FALSE) {
-		rb_debug ("not adding %s - %s - %s to  removable device since it's already present\n", title, album, artist);
+		rb_debug ("not adding %lu - %s - %s - %s to removable device since it's already present", track_number, title, album, artist);
 	}
 	return no_match;
 }
@@ -696,13 +727,14 @@ void
 rb_removable_media_source_track_added (RBRemovableMediaSource *source,
 				       RhythmDBEntry *entry,
 				       const char *uri,
+				       guint64 filesize,
 				       const char *mimetype)
 {
 	RBRemovableMediaSourceClass *klass = RB_REMOVABLE_MEDIA_SOURCE_GET_CLASS (source);
 	gboolean add_to_db = TRUE;
 
 	if (klass->impl_track_added)
-		add_to_db = klass->impl_track_added (source, entry, uri, mimetype);
+		add_to_db = klass->impl_track_added (source, entry, uri, filesize, mimetype);
 
 	if (add_to_db) {
 		RhythmDBEntryType entry_type;
@@ -742,3 +774,8 @@ rb_removable_media_source_track_add_error (RBRemovableMediaSource *source,
 	}
 }
 
+static char *
+impl_get_delete_action (RBSource *source)
+{
+	return g_strdup ("EditDelete");
+}
