@@ -1,7 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  arch-tag: Implementation of GtkTreeModel iface containing RBSource objects
- *
  * Copyright (C) 2003 Colin Walters <walters@verbum.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -9,6 +7,14 @@
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
+ * The Rhythmbox authors hereby grant permission for non-GPL compatible
+ * GStreamer plugins to be used and distributed together with GStreamer
+ * and Rhythmbox. This permission is above and beyond the permissions granted
+ * by the GPL license by which Rhythmbox is covered. If you modify this code
+ * you may extend this exception to your version of the code, but you are not
+ * obligated to do so. If you do not wish to do so, delete this exception
+ * statement from your version.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
@@ -35,6 +41,29 @@
 #include "rb-marshal.h"
 #include "rb-playlist-source.h"
 
+/**
+ * SECTION:rb-sourcelist-model
+ * @short_description: models backing the source list widget
+ *
+ * The #RBSourceList widget is backed by a #GtkTreeStore containing
+ * the sources and a set of attributes used to structure and display
+ * them, and a #GtkTreeModelFilter that hides sources with the
+ * visibility property set to FALSE.  This class implements the filter
+ * model and also creates the actual model.
+ *
+ * The source list model supports drag and drop in a variety of formats.
+ * The simplest of these are text/uri-list and application/x-rhythmbox-entry,
+ * which convey URIs and IDs of existing database entries.  When dragged
+ * to an existing source, these just add the URIs or entries to the target
+ * source.  When dragged to an empty space in the source list, this results
+ * in the creation of a static playlist.
+ *
+ * text/x-rhythmbox-artist, text/x-rhythmbox-album, and text/x-rhythmbox-genre
+ * are used when dragging items from the library browser.  When dragged to
+ * the source list, these result in the creation of a new auto playlist with
+ * the dragged items as criteria.
+ */
+
 struct RBSourceListModelPrivate
 {
 	gpointer dummy;
@@ -128,6 +157,15 @@ rb_sourcelist_model_class_init (RBSourceListModelClass *class)
 
 	o_class->finalize = rb_sourcelist_model_finalize;
 
+	/**
+	 * RBSourceListModel::drop-received:
+	 * @model: the #RBSourceListModel
+	 * @target: the #RBSource receiving the drop
+	 * @pos: the drop position
+	 * @data: the drop data
+	 *
+	 * Emitted when a drag and drop operation to the source list completes.
+	 */
 	rb_sourcelist_model_signals[DROP_RECEIVED] =
 		g_signal_new ("drop_received",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -164,6 +202,13 @@ rb_sourcelist_model_drag_source_init (RbTreeDragSourceIface *iface)
   iface->rb_drag_data_delete = rb_sourcelist_model_drag_data_delete;
 }
 
+/**
+ * rb_sourcelist_model_set_dnd_targets:
+ * @sourcelist: the #RBSourceListModel
+ * @treeview: the sourcel ist #GtkTreeView
+ *
+ * Sets up the drag and drop targets for the source list.
+ */
 void
 rb_sourcelist_model_set_dnd_targets (RBSourceListModel *sourcelist,
 				     GtkTreeView *treeview)
@@ -199,6 +244,14 @@ rb_sourcelist_model_finalize (GObject *object)
 	G_OBJECT_CLASS (rb_sourcelist_model_parent_class)->finalize (object);
 }
 
+/**
+ * rb_sourcelist_model_new:
+ *
+ * This constructs both the GtkTreeStore holding the source
+ * data and the filter model that hides invisible sources.
+ * 
+ * Return value: the #RBSourceListModel
+ */
 GtkTreeModel *
 rb_sourcelist_model_new (void)
 {
@@ -274,12 +327,14 @@ rb_sourcelist_model_drag_data_received (RbTreeDragDest *drag_dest,
 					GtkSelectionData *selection_data)
 {
 	RBSourceListModel *model;
+	GdkAtom type;
 
 	g_return_val_if_fail (RB_IS_SOURCELIST_MODEL (drag_dest), FALSE);
 	model = RB_SOURCELIST_MODEL (drag_dest);
+	type = gtk_selection_data_get_data_type (selection_data);
 
-	if (selection_data->type == gdk_atom_intern ("text/uri-list", TRUE) ||
-	    selection_data->type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE)) {
+	if (type == gdk_atom_intern ("text/uri-list", TRUE) ||
+	    type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE)) {
 		GtkTreeIter iter;
 		RBSource *target = NULL;
 
@@ -300,16 +355,16 @@ rb_sourcelist_model_drag_data_received (RbTreeDragDest *drag_dest,
 	}
 
         /* if artist, album or genre, only allow new playlists */
-        if (selection_data->type == gdk_atom_intern ("text/x-rhythmbox-album", TRUE) ||
-            selection_data->type == gdk_atom_intern ("text/x-rhythmbox-artist", TRUE) ||
-            selection_data->type == gdk_atom_intern ("text/x-rhythmbox-genre", TRUE)) {
+        if (type == gdk_atom_intern ("text/x-rhythmbox-album", TRUE) ||
+            type == gdk_atom_intern ("text/x-rhythmbox-artist", TRUE) ||
+            type == gdk_atom_intern ("text/x-rhythmbox-genre", TRUE)) {
                 rb_debug ("text/x-rhythmbox-(album|artist|genre) drag data received");
                 g_signal_emit (G_OBJECT (model), rb_sourcelist_model_signals[DROP_RECEIVED],
                                0, NULL, pos, selection_data);
                 return TRUE;
         }
 
-	if (selection_data->type == gdk_atom_intern ("application/x-rhythmbox-source", TRUE)) {
+	if (type == gdk_atom_intern ("application/x-rhythmbox-source", TRUE)) {
 		/* don't support dnd of sources */
 		return FALSE;
 	}
@@ -438,14 +493,16 @@ rb_sourcelist_model_drag_data_get (RbTreeDragSource *drag_source,
 {
 	char *path_str;
 	GtkTreePath *path;
+	GdkAtom selection_data_target;
 	guint target;
 
+	selection_data_target = gtk_selection_data_get_target (selection_data);
 	path = gtk_tree_row_reference_get_path (path_list->data);
 	if (path == NULL)
 		return FALSE;
 
 	if (!gtk_target_list_find (sourcelist_drag_target_list,
-				   selection_data->target,
+				   selection_data_target,
 				   &target)) {
 		return FALSE;
 	}
@@ -455,7 +512,7 @@ rb_sourcelist_model_drag_data_get (RbTreeDragSource *drag_source,
 		rb_debug ("getting drag data as rb source path");
 		path_str = gtk_tree_path_to_string (path);
 		gtk_selection_data_set (selection_data,
-					selection_data->target,
+					selection_data_target,
 					8, (guchar *) path_str,
 					strlen (path_str));
 		g_free (path_str);
@@ -509,7 +566,7 @@ rb_sourcelist_model_drag_data_get (RbTreeDragSource *drag_source,
 		g_object_unref (query_model);
 
 		gtk_selection_data_set (selection_data,
-					selection_data->target,
+					selection_data_target,
 					8, (guchar *) data->str,
 					data->len);
 
@@ -546,6 +603,20 @@ rb_sourcelist_model_row_deleted_cb (GtkTreeModel *model,
 	gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (sourcelist));
 }
 
+/**
+ * RBSourceListModelColumn:
+ * @RB_SOURCELIST_MODEL_COLUMN_PLAYING: TRUE if the source is playing
+ * @RB_SOURCELIST_MODEL_COLUMN_PIXBUF: the source's icon as a pixbuf
+ * @RB_SOURCELIST_MODEL_COLUMN_NAME: the source name
+ * @RB_SOURCELIST_MODEL_COLUMN_SOURCE: the #RBSource object
+ * @RB_SOURCELIST_MODEL_COLUMN_ATTRIBUTES: Pango attributes used to render the source name
+ * @RB_SOURCELIST_MODEL_COLUMN_VISIBILITY: the source's visibility
+ * @RB_SOURCELIST_MODEL_COLUMN_IS_GROUP: whether the row identifies a group or a source
+ * @RB_SOURCELIST_MODEL_COLUMN_GROUP_CATEGORY: if the row is a group, the category for the group
+ *
+ * Columns present in the source list model.
+ */
+
 /* This should really be standard. */
 #define ENUM_ENTRY(NAME, DESC) { NAME, "" #NAME "", DESC }
 
@@ -562,6 +633,8 @@ rb_sourcelist_model_column_get_type (void)
 			ENUM_ENTRY (RB_SOURCELIST_MODEL_COLUMN_SOURCE, "Source"),
 			ENUM_ENTRY (RB_SOURCELIST_MODEL_COLUMN_ATTRIBUTES, "Attributes"),
 			ENUM_ENTRY (RB_SOURCELIST_MODEL_COLUMN_VISIBILITY, "Visibility"),
+			ENUM_ENTRY (RB_SOURCELIST_MODEL_COLUMN_IS_GROUP, "Is group"),
+			ENUM_ENTRY (RB_SOURCELIST_MODEL_COLUMN_GROUP_CATEGORY, "Source group category"),
 			{ 0, 0, 0 }
 		};
 
