@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- * arch-tag: Implementation of main "Sources" display widget
  *
  * Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
  *
@@ -493,7 +491,8 @@ rb_sourcelist_expander_cell_data_func (GtkTreeViewColumn *column,
 		gboolean     row_expanded;
 
 		path = gtk_tree_model_get_path (model, iter);
-		row_expanded = gtk_tree_view_row_expanded (GTK_TREE_VIEW (column->tree_view), path);
+		row_expanded = gtk_tree_view_row_expanded (GTK_TREE_VIEW (gtk_tree_view_column_get_tree_view (column)),
+							   path);
 		gtk_tree_path_free (path);
 
 		g_object_set (cell,
