@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of main song information display widget
  *
  *  Copyright (C) 2002, 2003 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
@@ -306,7 +304,7 @@ rb_header_playing_song_changed_cb (RBShellPlayer *player, RhythmDBEntry *entry,
 		header->priv->duration = 0;
 	}
 
-	header->priv->adjustment->upper = header->priv->duration;
+	gtk_adjustment_set_upper (header->priv->adjustment, header->priv->duration);
 	gtk_adjustment_changed (header->priv->adjustment);
 
 	rb_header_sync (header);
@@ -484,9 +482,16 @@ rb_header_sync (RBHeader *header)
 
 		label_str = g_string_sized_new (100);
 
-		/* stick a right-to-left mark in the string for RTL display */
+		/* force the string to have the same direction as the widget.
+		 * this fixes the display when the head of the text (usually
+		 * the track name) has a different direction than the widget.
+		 */
 		if (gtk_widget_get_direction (GTK_WIDGET (header->priv->song)) == GTK_TEXT_DIR_RTL) {
+			/* insert a unicode RLM */
 			g_string_append (label_str, "\xE2\x80\x8F");
+		} else {
+			/* insert a unicode LRM */
+			g_string_append (label_str, "\xE2\x80\x8E");
 		}
 
 		append_and_free (label_str, TITLE_MARKUP (title));
