@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Header for main song information display widget
- *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
  *
@@ -67,10 +65,6 @@ GType		rb_header_get_type		(void);
 RBHeader *	rb_header_new			(RBShellPlayer *shell_player,
 						 RhythmDB *db);
 
-void		rb_header_set_playing_entry	(RBHeader *header,
-						 RhythmDBEntry *entry,
-						 gboolean seekable);
-
 void		rb_header_set_show_position_slider (RBHeader *header,
                                                     gboolean show);
 
