@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of library browser widget
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
@@ -37,6 +35,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
+#include "gseal-gtk-compat.h"
 #include "rb-library-browser.h"
 #include "rb-preferences.h"
 #include "eel-gconf-extensions.h"
@@ -399,7 +398,7 @@ update_browser_property_visibilty (RhythmDBPropType prop,
 {
 	gboolean old_vis, new_vis;
 
-	old_vis = GTK_WIDGET_VISIBLE (view);
+	old_vis = gtk_widget_get_visible (GTK_WIDGET (view));
 	new_vis = (g_list_find (properties, (gpointer)prop) != NULL);
 
 	if (old_vis != new_vis) {
