@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of RhythmDB query creation dialog
  *
  *  Copyright (C) 2003, 2004 Colin Walters <walters@gnome.org>
  *
@@ -181,6 +179,7 @@ rb_query_creator_constructed (GObject *object)
 	RBQueryCreatorPrivate *priv;
 	RBQueryCreator *creator;
 	GtkWidget *mainbox;
+	GtkWidget *content_area;
 	GtkBuilder *builder;
 
 	RB_CHAIN_GOBJECT_METHOD (rb_query_creator_parent_class, constructed, object);
@@ -213,8 +212,10 @@ rb_query_creator_constructed (GObject *object)
 	else
 		gtk_window_set_title (GTK_WINDOW (creator), _("Edit Automatic Playlist"));
 
+	content_area = gtk_dialog_get_content_area (GTK_DIALOG (creator));
+
 	gtk_container_set_border_width (GTK_CONTAINER (creator), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (creator)->vbox), 2);
+	gtk_box_set_spacing (GTK_BOX (content_area), 2);
 	gtk_dialog_set_has_separator (GTK_DIALOG (creator), FALSE);
 
 	builder = rb_builder_load ("create-playlist.ui", creator);
@@ -245,7 +246,7 @@ rb_query_creator_constructed (GObject *object)
 		append_row (creator);
 
 	mainbox = GTK_WIDGET (gtk_builder_get_object (builder, "complex-playlist-creator"));
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (creator)->vbox), mainbox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (content_area), mainbox, FALSE, FALSE, 0);
 	gtk_widget_show_all (GTK_WIDGET (creator));
 
 	g_object_unref (builder);
@@ -914,7 +915,7 @@ create_property_option_menu (RBQueryCreator *creator,
 
 	combo = gtk_combo_box_new_text ();
 	for (i = 0; i < length; i++) {
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _(options[i].name));
+		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), g_dpgettext2 (NULL, "query-criteria", options[i].name));
 	}
 
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
@@ -969,7 +970,7 @@ setup_sort_option_menu (RBQueryCreator *creator,
 		GtkTreeIter iter;
 
 		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter, 0, _(options[i].name), -1);
+		gtk_list_store_set (store, &iter, 0, g_dpgettext2 (NULL, "query-sort", options[i].name), -1);
 	}
 
 	gtk_combo_box_set_model (GTK_COMBO_BOX (option_menu), GTK_TREE_MODEL (store));
