@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of search entry widget
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
@@ -10,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -27,7 +33,6 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include "libsexy/sexy-icon-entry.h"
 
 #include "rb-search-entry.h"
 
@@ -42,6 +47,10 @@ static void rb_search_entry_activate_cb (GtkEntry *gtkentry,
 static gboolean rb_search_entry_focus_out_event_cb (GtkWidget *widget,
 				                    GdkEventFocus *event,
 				                    RBSearchEntry *entry);
+static void rb_search_entry_clear_cb (GtkEntry *entry,
+				      GtkEntryIconPosition icon_pos,
+				      GdkEvent *event,
+				      RBSearchEntry *search_entry);
 
 struct RBSearchEntryPrivate
 {
@@ -57,6 +66,21 @@ struct RBSearchEntryPrivate
 G_DEFINE_TYPE (RBSearchEntry, rb_search_entry, GTK_TYPE_HBOX)
 #define RB_SEARCH_ENTRY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_SEARCH_ENTRY, RBSearchEntryPrivate))
 
+/**
+ * SECTION:rb-search-entry
+ * @short_description: text entry widget for the search box
+ *
+ * The search entry contains a label and a text entry box.
+ * The text entry box contains an icon that acts as a 'clear'
+ * button.
+ *
+ * Signals are emitted when the search text changes,
+ * arbitrarily rate-limited to one every 300ms.
+ *
+ * When the text entry widget is non-empty, its colours are
+ * changed to display the text in black on yellow.
+ */
+
 enum
 {
 	SEARCH,
@@ -73,6 +97,15 @@ rb_search_entry_class_init (RBSearchEntryClass *klass)
 
 	object_class->finalize = rb_search_entry_finalize;
 
+	/**
+	 * RBSearchEntry::search:
+	 * @entry: the #RBSearchEntry
+	 * @text: search text
+	 *
+	 * Emitted when the search text changes.  A signal
+	 * handler must initiate a search on the current
+	 * source.
+	 */
 	rb_search_entry_signals[SEARCH] =
 		g_signal_new ("search",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -83,15 +116,24 @@ rb_search_entry_class_init (RBSearchEntryClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_STRING);
+
+	/**
+	 * RBSearchEntry::activate:
+	 * @entry: the #RBSearchEntry
+	 * @text: search text
+	 *
+	 * Emitted when the entry is activated.
+	 */
 	rb_search_entry_signals[ACTIVATE] =
 		g_signal_new ("activate",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RBSearchEntryClass, activate),
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
+			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE,
-			      0);
+			      1,
+			      G_TYPE_STRING);
 
 	g_type_class_add_private (klass, sizeof (RBSearchEntryPrivate));
 }
@@ -116,8 +158,17 @@ rb_search_entry_init (RBSearchEntry *entry)
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
 	gtk_box_pack_start (GTK_BOX (entry), label, FALSE, TRUE, 0);
 
-	entry->priv->entry = sexy_icon_entry_new ();
-	sexy_icon_entry_add_clear_button (SEXY_ICON_ENTRY (entry->priv->entry));
+	entry->priv->entry = gtk_entry_new ();
+	gtk_entry_set_icon_from_stock (GTK_ENTRY (entry->priv->entry),
+				       GTK_ENTRY_ICON_SECONDARY,
+				       GTK_STOCK_CLEAR);
+	gtk_entry_set_icon_tooltip_text (GTK_ENTRY (entry->priv->entry),
+					 GTK_ENTRY_ICON_SECONDARY,
+					 _("Clear the search text"));
+	g_signal_connect_object (GTK_ENTRY (entry->priv->entry),
+				 "icon-press",
+				 G_CALLBACK (rb_search_entry_clear_cb),
+				 entry, 0);
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label),
 				       entry->priv->entry);
@@ -153,6 +204,11 @@ rb_search_entry_finalize (GObject *object)
 	G_OBJECT_CLASS (rb_search_entry_parent_class)->finalize (object);
 }
 
+/**
+ * rb_search_entry_new:
+ *
+ * Return value: new search entry widget.
+ */
 RBSearchEntry *
 rb_search_entry_new (void)
 {
@@ -167,6 +223,13 @@ rb_search_entry_new (void)
 	return entry;
 }
 
+/**
+ * rb_search_entry_clear:
+ * @entry: a #RBSearchEntry
+ *
+ * Clears the search entry text.  The 'search' signal will
+ * be emitted.
+ */
 void
 rb_search_entry_clear (RBSearchEntry *entry)
 {
@@ -182,6 +245,14 @@ rb_search_entry_clear (RBSearchEntry *entry)
 	entry->priv->clearing = FALSE;
 }
 
+/**
+ * rb_search_entry_set_text:
+ * @entry: a #RBSearchEntry
+ * @text: new search text
+ *
+ * Sets the text in the search entry box.
+ * The 'search' signal will be emitted.
+ */
 void
 rb_search_entry_set_text (RBSearchEntry *entry, const char *text)
 {
@@ -192,24 +263,22 @@ rb_search_entry_set_text (RBSearchEntry *entry, const char *text)
 static void
 rb_search_entry_check_style (RBSearchEntry *entry)
 {
-	/*GdkColor fg_colour;*/
-	GdkColor bg_colour;
-	static const GdkColor fallback_bg_colour = { 0, 0xf7f7, 0xf7f7, 0xbebe }; /* yellow-ish */
+	static const GdkColor bg_colour = { 0, 0xf7f7, 0xf7f7, 0xbebe }; /* yellow-ish */
+	static const GdkColor fg_colour = { 0, 0, 0, 0 }; /* black. */
 	const gchar* text;
 
 	if (entry->priv->is_a11y_theme)
 		return;
 
-	/*fg_colour = GTK_WIDGET(entry)->style->text[GTK_STATE_NORMAL];*/
-	bg_colour = fallback_bg_colour;
-
 	text = gtk_entry_get_text (GTK_ENTRY (entry->priv->entry));
 	if (text && *text) {
-		/*gtk_widget_modify_text (entry->priv->entry, GTK_STATE_NORMAL, &fg_colour);*/
+		gtk_widget_modify_text (entry->priv->entry, GTK_STATE_NORMAL, &fg_colour);
 		gtk_widget_modify_base (entry->priv->entry, GTK_STATE_NORMAL, &bg_colour);
+		gtk_widget_modify_cursor (entry->priv->entry, &fg_colour, &fg_colour);
 	} else {
-		/*gtk_widget_modify_text (entry->priv->entry, GTK_STATE_NORMAL, NULL);*/
+		gtk_widget_modify_text (entry->priv->entry, GTK_STATE_NORMAL, NULL);
 		gtk_widget_modify_base (entry->priv->entry, GTK_STATE_NORMAL, NULL);
+		gtk_widget_modify_cursor (entry->priv->entry, NULL, NULL);
 	}
 
 	gtk_widget_queue_draw (GTK_WIDGET (entry));
@@ -267,8 +336,14 @@ rb_search_entry_focus_out_event_cb (GtkWidget *widget,
 	return FALSE;
 }
 
+/**
+ * rb_search_entry_searching:
+ * @entry: a #RBSearchEntry
+ *
+ * Return value: TRUE if there is search text
+ */
 gboolean
-rb_search_entry_searching(RBSearchEntry *entry)
+rb_search_entry_searching (RBSearchEntry *entry)
 {
 	return strcmp ("", gtk_entry_get_text (GTK_ENTRY (entry->priv->entry))) != 0;
 }
@@ -277,11 +352,27 @@ static void
 rb_search_entry_activate_cb (GtkEntry *gtkentry,
 			     RBSearchEntry *entry)
 {
-	g_signal_emit (G_OBJECT (entry), rb_search_entry_signals[ACTIVATE], 0);
+	g_signal_emit (G_OBJECT (entry), rb_search_entry_signals[ACTIVATE], 0,
+		       gtk_entry_get_text (GTK_ENTRY (entry->priv->entry)));
 }
 
+/**
+ * rb_search_entry_grab_focus:
+ * @entry: a #RBSearchEntry
+ *
+ * Grabs input focus for the text entry widget.
+ */
 void
 rb_search_entry_grab_focus (RBSearchEntry *entry)
 {
 	gtk_widget_grab_focus (GTK_WIDGET (entry->priv->entry));
 }
+
+static void
+rb_search_entry_clear_cb (GtkEntry *entry,
+			  GtkEntryIconPosition icon_pos,
+			  GdkEvent *event,
+			  RBSearchEntry *search_entry)
+{
+	rb_search_entry_set_text (search_entry, "");
+}
